# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

""" Schema test """

from cubicweb.devtools.testlib import CubicWebTC

def create_subject_container_data(req):
    """ Several entities involving composite relations are created,
        according to the schema.
    """
    t_study = req.create_entity('Study', name=u'Test study',
                                data_filepath=u'test/data/filepath')
    t_card = req.create_entity('Card', title=u'This is a card.',
                               reverse_wiki=t_study)
    t_subj = req.create_entity('Subject', identifier=u'test_subj',
                               gender=u'unknown', handedness=u'mixed')
    t_adm = req.create_entity('Admission', admission_of=t_subj,
                              admission_in=t_study)
    t_step = req.create_entity('AdmissionStep', step_of=t_adm, name=u'step')
    t_ther = req.create_entity('Therapy', reverse_related_therapies=t_subj)
    t_body = req.create_entity('BodyLocation', identifier=u'bd', name=u'bd')
    t_disease = req.create_entity('Disease', identifier=u'di', name=u'di')
    t_diag = req.create_entity('Diagnostic', reverse_related_diagnostics=t_subj,
                               diagnostic_location=t_body,
                               diagnosed_disease=t_disease)
    t_center = req.create_entity('Center', identifier=u'Test center ID',
                                 name=u'Test center name')
    t_assessment = req.create_entity('Assessment',
                                     related_study=t_study,
                                     reverse_holds=t_center,
                                     reverse_concerned_by=t_subj)
    t_questionnaire = req.create_entity('Questionnaire', name=u'Test quiz',
                                        identifier=u'test_quiz_id',
                                        type=u'test')
    t_qrun = req.create_entity('QuestionnaireRun',
                               user_ident=u'test_qrun_uident',
                               instance_of=t_questionnaire,
                               reverse_generates=t_assessment,
                               concerns=t_subj,
                               related_study=t_study)
    t_comm = req.create_entity('Comment', content=u'This is a test comment',
                               comments=t_qrun)
    t_score_def = req.create_entity('ScoreDefinition', name=u'Test score def',
                                    type=u'string')
    t_score_val = req.create_entity('ScoreValue', definition=t_score_def,
                                    text=u'test score val', measure=t_qrun)

class BrainomicsSchemaTC(CubicWebTC):
    """ Test proper behavior with respect to the composite relations. """

    def setup_database(self):
        """ Several entities involving composite relations are created,
            according to the schema.
        """
        req = self.request()
        create_subject_container_data(req)

    def test_cleanup_on_qrun_delete(self):
        """ Test that on QuestionnaireRun deletion, the Comment,
            ExternalResource and ScoreValue are deleted, but the Subject,
            Assessment and Study are not deleted.
        """
        req = self.request()
        qrun_eid = req.execute('Any X WHERE X is QuestionnaireRun').get_entity(0, 0).eid
        req.execute('Delete QuestionnaireRun X WHERE X eid %(qreid)s', {'qreid': qrun_eid})
        self.commit()
        db_comm = req.execute('Any X WHERE X is Comment')
        if db_comm:
            self.fail('The Comment was not deleted')
        db_ext_res = req.execute('Any X WHERE X is ExternalFile')
        if db_ext_res:
            self.fail('The ExternalResource was not deleted')
        db_score_val = req.execute('Any X WHERE X is ScoreValue')
        if db_score_val:
            self.fail('The ScoreValue was not deleted')
        db_subj = req.execute('Any X WHERE X is Subject')
        if not db_subj:
            self.fail('The Subject was deleted')
        db_assess = req.execute('Any X WHERE X is Assessment')
        if not db_assess:
            self.fail('The Assessment was deleted')
        db_study = req.execute('Any X WHERE X is Study')
        if not db_study:
            self.fail('The Study was deleted')

    def test_cleanup_on_study_delete(self):
        """ Test that on Study deletion, the QuestionnaireRun and Card are deleted.
        """
        req = self.request()
        study_eid = req.execute('Any X WHERE X is Study').get_entity(0, 0).eid
        db_card = req.execute('Any X WHERE X is Card, X title "This is a card."')
        if not db_card:
            self.fail('The test Card was not founs')
        # Delete Assessment first:
        req.execute('DELETE Assessment X WHERE X related_study Y, Y eid %(studeid)s',
                    {'studeid': study_eid})
        # Also delete Admission, for cardinality constraints reasons:
        req.execute('DELETE Admission X WHERE X admission_in Y, Y eid %(studeid)s',
                    {'studeid': study_eid})
        req.execute('DELETE Study X WHERE X eid %(studeid)s', {'studeid': study_eid})
        self.commit()
        db_qrun = req.execute('Any X WHERE X is QuestionnaireRun')
        if db_qrun:
            self.fail('The QuestionnaireRun was not deleted')
        db_card = req.execute('Any X WHERE X is Card, X title "This is a card."')
        if db_card:
            self.fail('The Card was not deleted')

    def test_cleanup_on_assessment_delete(self):
        """ Test that on Assessment deletion, the QuestionnaireRun is deleted, but
            the Study is not deleted.
        """
        req = self.request()
        assess_eid = req.execute('Any X WHERE X is Assessment').get_entity(0, 0).eid
        db_qrun = req.execute('Any X WHERE X is QuestionnaireRun')
        if not db_qrun:
            self.fail('No QuestionnaireRun was found')
        req.execute('DELETE Assessment X WHERE X eid %(aseid)s', {'aseid': assess_eid})
        self.commit()
        db_qrun = req.execute('Any X WHERE X is QuestionnaireRun')
        if db_qrun:
            self.fail('The QuestionnaireRun was not deleted')
        db_study = req.execute('Any X WHERE X is Study')
        if not db_study:
            self.fail('The Study was deleted')


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
from logilab.common.decorators import monkeypatch
from logilab.mtconverter import xml_escape
import warnings

from cubicweb import ObjectNotFound
from cubicweb.predicates import is_instance, nonempty_rset
from cubicweb.utils import make_uid
from cubicweb.selectors import is_instance
from cubicweb.web.views.ajaxcontroller import ajaxfunc
from cubicweb.appobject import AppObject
from cubicweb.view import View
from cubicweb.web import component, facet as facetbase
from cubicweb.web.facet import FilterRQLBuilder
from cubicweb.web.views import facets

try:
    from cubicweb.web.facet import AbstractRangeRQLPathFacet
    warnings.warn('AbstractRangeRQLPathFacet are now available. '
                  'The file cubes.brainomics.views.rqlpathfacets should be removed')
except ImportError:
    from cubes.brainomics.views.rqlpathfacets import AbstractRangeRQLPathFacet
    warnings.warn('AbstractRangeRQLPathFacet are backport from CW 3.19')


@monkeypatch(facetbase.FilterRQLBuilder)
def build_rql(self):
    form = self._cw.form
    facetids = form['facets']
    # start brainomincs customization
    facetids = facetids.split(',') if isinstance(facetids, basestring) else facetids
    # end brainomincs customization
    # XXX Union unsupported yet
    select = self._cw.vreg.parse(self._cw, form['baserql']).children[0]
    filtered_variable = facetbase.get_filtered_variable(select, form.get('mainvar'))
    toupdate = []
    for facetid in facetids:
        facet = facetbase.get_facet(self._cw, facetid, select, filtered_variable)
        facet.add_rql_restrictions()
        if facet.needs_update:
            toupdate.append(facetid)
    return  select.as_string(), toupdate


class DynamicFacetView(View):
    __regid__ = 'dynamic-facet'

    def call(self):
        vocab = self._cw.form['vocabulary']
        if not vocab:
            return
        rql = self._cw.form['rql']
        vocab = (vocab,) if isinstance(vocab, basestring) else vocab
        facetid = self._cw.form['dynamic-facet-id']
        select = self._cw.vreg.parse(self._cw, rql).children[0]
        rset = self._cw.execute(rql)
        filtered_variable = facetbase.get_filtered_variable(select, None)
        facetbase.prepare_select(select, filtered_variable)
        builder = self._cw.vreg['dynamic-facet-builder'].select_or_none(facetid, self._cw)
        if builder:
            for voc in vocab:
                try:
                    _facet = facetbase.get_facet(self._cw, facetid+voc, select, filtered_variable)
                except ObjectNotFound:
                    _facet = builder.build_facet(voc, rset, select, filtered_variable)
                wdg = _facet.get_widget()
                if wdg:
                    wdg.w = self.w
                    wdg._render()
                    self.w(u'<input type="hidden" name="facets" value="%s" />' %
                           xml_escape(_facet.__regid__))
                else:
                    self.warning('DynamicFacetView : no widget for facet "%s"' % _facet)




class FloatFacetRangeWidget(facetbase.FacetRangeWidget):
    onload = u'''
    var _formatter = %(formatter)s;
    jQuery("#%(sliderid)s").slider({
        range: true,
        min: %(minvalue)s,
        max: %(maxvalue)s,
        step: 0.01,
        values: [%(minvalue)s, %(maxvalue)s],
        stop: function(event, ui) { // submit when the user stops sliding
           var form = $('#%(sliderid)s').closest('form');
           buildRQL.apply(null, cw.evalJSON(form.attr('cubicweb:facetargs')));
        },
        slide: function(event, ui) {
            jQuery('#%(sliderid)s_inf').html(_formatter(ui.values[0]));
            jQuery('#%(sliderid)s_sup').html(_formatter(ui.values[1]));
            jQuery('input[name="%(facetname)s_inf"]').val(ui.values[0]);
            jQuery('input[name="%(facetname)s_sup"]').val(ui.values[1]);
        }
   });
   // use JS formatter to format value on page load
   jQuery('#%(sliderid)s_inf').html(_formatter(jQuery('input[name="%(facetname)s_inf"]').val()));
   jQuery('#%(sliderid)s_sup').html(_formatter(jQuery('input[name="%(facetname)s_sup"]').val()));
'''

    #'# make emacs happier
class DynamicTextInputRangeFacet(facetbase.RangeFacet):
    __abstract__ = True
    wdgclass = FloatFacetRangeWidget
    needs_update = True


class AbstractDynamicRangeFacetBuilder(AppObject):
    __registry__ = 'dynamic-facet-builder'
    __abstract__ = True
    filter_variable = None

    def get_title(self, vocid):
        raise NotImplementedError

    def get_path(self, vocid):
        raise NotImplementedError

    def build_facet(self, vocid, rset, select, filtered_variable):
        class DynamicFacet(AbstractRangeRQLPathFacet, DynamicTextInputRangeFacet):
            __regid__ = self.__regid__ + vocid
            path = self.get_path(vocid)
            filter_variable = self.filter_variable
            title = self.get_title(vocid)
            dynamic_facet = True

        # register the newly created class in facets
        vreg = self._cw.vreg['facets']
        vreg.register(DynamicFacet)
        # register cw_property_defs
        DynamicFacet.__registered__(vreg)

        # Facet rendering
        _facet = DynamicFacet(self._cw, select, filtered_variable)
        _facet.cw_rset = rset
        return _facet


class AbstractDynamicFacetBox(component.Component):
    __registry__ = 'dynamic-facet'
    __abstract__ = True
    title = None
    __select__ = component.Component.__select__  & nonempty_rset()

    def get_vocabulary(self):
        raise NotImplementedError

    def render(self, w, **kwargs):
        w(u'<h5>%s</h5>' % self._cw._(self.title))
        w(u'<select id="%s" multiple>' % self.__regid__)
        for eid, name in self.get_vocabulary():
            w(u'<option name=%s value=%s> %s </option>' % (name, eid, name))
        w(u'</select>')
        w(u'''<button onclick="cw.cubes.brainomics.addDynamicFacet('%s');return false;"
              class="btn btn-default">%s</button>''' % (self.__regid__, self._cw._('Filter')))




# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-brainomics views/forms/actions/components for web ui"""
from logilab.mtconverter import xml_escape
from cubicweb.selectors import is_instance
from cubicweb.web.views.tableview import (TableLayout, EntityTableView,
                                          RelationColRenderer, EntityTableColRenderer)

from cubes.brainomics.views.slickgrid import SlickGridEntityTableView

_ = unicode


class FilterTableLayout(TableLayout):
    __regid__ = 'filter_table_layout'
    display_filter = 'top'
    hide_filter = False


def render_width(w, entity):
    w(unicode(entity.genomic_region[0].width))


class AbstractCghTableView(SlickGridEntityTableView, EntityTableView):
    __abtract__ = True
    paginable = False
    __select__ = EntityTableView.__select__ & is_instance('CghResult')
    columns = ['cgh_ratio', 'log2_ratio', 'status', 'width', 'numprobes']
    column_renderers = {'width':  EntityTableColRenderer(renderfunc=render_width)}
    grid_options = {'enableCellNavigation': True, 'frozenColumn': 0}
    default_column_options = {'sortable': True, 'width': 140,}



class CghTableView(AbstractCghTableView):
    __regid__ = 'genmeas-table-view'
    columns = ['genomic_region',] + AbstractCghTableView.columns
    AbstractCghTableView.column_renderers.update({'genomic_region':
                                                  RelationColRenderer(role='subject')})



class GeneCghTableView(AbstractCghTableView):
    __regid__ = 'gene-genmeas-table-view'
    columns = ['genomic_region', 'related_measure'] + AbstractCghTableView.columns
    AbstractCghTableView.column_renderers.update({'related_measure':
                                                  RelationColRenderer(role='subject',
                                                                      vid='incontext'),
                                                  'genomic_region':
                                                  RelationColRenderer(role='subject',
                                                                      vid='incontext')})


class RegionCghTableView(AbstractCghTableView):
    __regid__ = 'region-genmeas-table-view'
    columns = ['related_measure',] + AbstractCghTableView.columns
    AbstractCghTableView.column_renderers.update({'related_measure':
                                                  RelationColRenderer(role='subject',
                                                                      vid='incontext')})


class AbstractMutationTableView(SlickGridEntityTableView, EntityTableView):
    __abstract__ = True
    paginable = False
    __select__ = EntityTableView.__select__ & is_instance('Mutation')

    columns = ['conclusions', 'position_in_gene',
               'mutation_type', 'amplicon', 'protein',
               'classification_type', 'variant_frequency',
               'reference_base', 'variant_base', 'variant_coverage', 'coverage',
               'reference_id', 'dbsnp', 'locus_version', 'nucl',
               'sift', 'polyphen', 'classification_variant']


class MutationTableView(AbstractMutationTableView):
    __regid__ = 'genmeas-table-view'
    column_renderers = {'related_gene': RelationColRenderer(role='subject', vid='incontext')}
    columns = ['related_gene',] + AbstractMutationTableView.columns


class GeneMutationTableView(AbstractMutationTableView):
    __regid__ = 'gene-genmeas-table-view'
    column_renderers = {'related_measure': RelationColRenderer(role='subject', vid='incontext')}
    columns = ['related_measure',] + AbstractMutationTableView.columns


###############################################################################
### SCORE GROUP ###############################################################
###############################################################################
class ScoreValueOutofContextTableView(EntityTableView):
    __select__ = EntityTableView.__select__ & is_instance('ScoreValue')
    __regid__ = 'scorevalue-outofcontext-table-view'

    columns = ['definition', 'text', 'value']
    column_renderers = {'definition': RelationColRenderer(role='subject')}


class ScoreValueInContextTableView(EntityTableView):
    __select__ = EntityTableView.__select__ & is_instance('ScoreValue')
    __regid__ = 'scorevalue-incontext-table-view'

    def render_value(w, entity):
        w(u'<ul>%s</ul>' % entity.complete_value)

    def render_datetime(w, entity):
        w(u'<ul>%s</ul>' % (entity.datetime or u'-'))

    def render_target(w, entity):
        w(u'%s' % entity.subject.view('incontext'))

    columns = ['subject', 'value', 'datetime']
    column_renderers = {'datetime': EntityTableColRenderer(renderfunc=render_datetime),
                        'subject': EntityTableColRenderer(renderfunc=render_target),
                        'value': EntityTableColRenderer(renderfunc=render_value)}


###############################################################################
### DRUG TAKES ################################################################
###############################################################################
class AbstractDrugTakeTableView(SlickGridEntityTableView):
    __abstract__ = True
    paginable = False
    __select__ = EntityTableView.__select__ & is_instance('DrugTake')
    grid_options = {'enableCellNavigation': True, 'frozenColumn': 0}
    default_column_options = {'sortable': True, 'width': 140,}
    columns = ['drug']
    column_renderers = {'drug': RelationColRenderer(role='subject', vid='incontext')}


class InTherapyDrugTakeTableView(AbstractDrugTakeTableView):
    __regid__ = 'intherapy-drugtake-table-view'

    columns = (AbstractDrugTakeTableView.columns +
               ['start_taking_date', 'stop_taking_date',
                'take_order', 'dosis', 'unit', 'number_of_cycles', 'dosis_percentage',
                'reduced_dosis'])


class DrugTakeTableView(AbstractDrugTakeTableView):
    __regid__ = 'drugtake-table-view'

    columns = (AbstractDrugTakeTableView.columns +
               ['taken_in_therapy', 'start_taking_date', 'stop_taking_date',
                'take_order', 'dosis', 'unit', 'number_of_cycles', 'dosis_percentage',
                'reduced_dosis'])

    column_renderers = AbstractDrugTakeTableView.column_renderers
    column_renderers['taken_in_therapy'] = RelationColRenderer(role='subject', vid='incontext')


###############################################################################
### ANSWERS ###################################################################
###############################################################################
class AnswerInContextTableView(EntityTableView):
    __select__ = EntityTableView.__select__ & is_instance('Answer')
    __regid__ = 'answer-incontext-table-view'
    columns = ['answer', 'value', 'datetime']

    def render_value(w, entity):
        w(u'%s' % entity.computed_value)

    def render_text(w, entity):
        w(u'%s' % entity.question[0].text)

    def render_answer(w, entity):
        w(u'<a href="%s">%s</a>' % (xml_escape(entity.absolute_url()),
                                    xml_escape(entity.dc_title())))

    column_renderers = {'answer': EntityTableColRenderer(renderfunc=render_answer),
                        'value': EntityTableColRenderer(renderfunc=render_value)}


class AnswerOutOfContextTableView(EntityTableView):
    __select__ = EntityTableView.__select__ & is_instance('Answer')
    __regid__ = 'answer-outofcontext-table-view'
    columns = ['question', 'text', 'value', 'datetime']

    def render_text(w, entity):
        w(u'%s' % entity.question[0].text)

    def render_question(w, entity):
        entity = entity.question[0]
        w(u'<a href="%s">%s</a>' % (xml_escape(entity.absolute_url()),
                                    xml_escape(entity.dc_title())))

    column_renderers = {'question': EntityTableColRenderer(renderfunc=render_question),
                        'text': EntityTableColRenderer(renderfunc=render_text)}


###############################################################################
### QUESTIONS #################################################################
###############################################################################
class QuestionTableView(EntityTableView):
    __select__ = EntityTableView.__select__ & is_instance('Question')
    __regid__ = 'question-table-view'
    columns = ['question', 'text', 'type', 'possible_answers', 'answers']

    def render_question(w, entity):
        w(u'%s'% xml_escape(entity.dc_title()))

    def render_answers(w, entity):
        w(u'<a href="%s">%s</a>' % (xml_escape(entity.absolute_url()),
                                    _('See detailed question')))

    column_renderers = {'question': EntityTableColRenderer(renderfunc=render_question),
                        'answers': EntityTableColRenderer(renderfunc=render_answers)}


###############################################################################
### DIAGNOSTIC/ANALYSIS #######################################################
###############################################################################
class DiagnosticsTableView(EntityTableView):
    __select__ = EntityTableView.__select__ & is_instance('Diagnostic')
    __regid__ = 'diagnostics-table-view'

    columns = ['diagnostic_date', 'conclusion', 'diagnostic_location',
               'diagnosed_disease', 'based_on']
    column_renderers = {'diagnostic_location': RelationColRenderer(role='subject', vid='incontext'),
                        'diagnosed_disease': RelationColRenderer(role='subject', vid='incontext'),
                        'technique_type': RelationColRenderer(role='subject', vid='incontext'),
                        'based_on': RelationColRenderer(role='subject', vid='incontext'),
                       }


class TechnicalAnalysisTableView(EntityTableView):
    __select__ = EntityTableView.__select__ & is_instance('TechnicalAnalysis')
    __regid__ = 'analysis-table-view'

    def render_analysis_on(w, entity):
        if entity.performed_on:
            return entity.performed_on[0].view('incontext')
        elif entity.analysis_of_sample:
            return entity.analysis_of_sample[0].view('incontext')

    columns = ['protocols', 'technique_type', 'results', 'conclusion',
               'analysis_date', _('analysis_on')]
    column_renderers = {'protocols': RelationColRenderer(role='subject', vid='incontext'),
                        'technique_type': RelationColRenderer(role='subject', vid='incontext'),
                        'analysis_on': EntityTableColRenderer(renderfunc=render_analysis_on)
                       }


###############################################################################
### CLINIPATH #################################################################
###############################################################################
class BiopsySampleTableView(EntityTableView):
    __select__ = EntityTableView.__select__ & is_instance('BiopsySample')
    __regid__ = 'biopsysample-table-view'
    layout_id = 'filter_table_layout'
    columns = ['size', 'name', 'code', 'tumoral_cell_percent', 'necrosis_percent']



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-suivimp views/forms/actions/components for web ui"""
from logilab.common.decorators import monkeypatch

from cubicweb.web.views import basetemplates
from cubicweb.web.views import uicfg


###############################################################################
### UICFG MODIFICATIONS #######################################################
###############################################################################
uicfg.primaryview_display_ctrl._relrsetdefs = {}
uicfg.primaryview_display_ctrl._attrrsetdefs = {}

@monkeypatch(uicfg.DisplayCtrlRelationTags)
def __init__(self, *args, **kwargs):
    super(DisplayCtrlRelationTags, self).__init__(*args, **kwargs)
    self.counter = 0
    self._attrrsetdefs = {}
    self._relrsetdefs = {}

@monkeypatch(uicfg.DisplayCtrlRelationTags)
def display_rset(self, etype, section, props):
    """
    The display_rset method of uicfg take 3 arguments:

        * the name of the etype concerned by the rule;

        * the section where the information is displayed (attributes or relations);

        * a dictionnary of properties.

    The dictionnary of properties must have:

       * a 'callback' item OR a 'rql' and a 'vid' items.

    The 'callback' is a function that takes the entity and render an HTML snipet.

    The 'rql' is a rql query where the variable 'X' is the current entity. The 'vid'
    is the __regid__ of the view that will be applied to the result of the rql query.

    It could also have:

       * a 'label' ('' if not given);

       * an 'order' (9999 if not given);

    It should be used as follows, e.g. using an etype property:

      >>> _pvdc = uicfg.primaryview_display_ctrl
      >>> _pvdc.display_rset('MyEtype', 'attributes',
                             {'callback': lambda x: x.formatted_description,
                              'label': _('description')})

    or with a 'rql' and 'vid' attributes (in this case 'X' is the current entity):

      >>> _pvdc = uicfg.primaryview_display_ctrl
      >>> _pvdc.display_rset('MyEtype', 'relations',
                             {'rql': 'Any Z WHERE X relation1 Y, Y relation2 Z',
                              'vid': 'my-view',
                              'label': _('My label')})

    or with a 'callback' using a view:


      >>> _pvdc = uicfg.primaryview_display_ctrl
      >>> _pvdc.display_rset('MyEtype', 'relations',
                             {'callback': lambda x: x.view('my-secondary-view'),
                             'order': 2,
                             'label': _('My label')})

    """
    assert ('rql' in props and 'vid' in props) or 'callback' in props
    if section == 'attributes':
        self._attrrsetdefs.setdefault(etype, []).append(props)
    elif section == 'relations':
        self._relrsetdefs.setdefault(etype, []).append(props)
    else:
        self.warning('display_rset for %s should be in section '
                     '"attributes" or "relations"' % etype)




# copyright 2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
from cubicweb.devtools.testlib import CubicWebTC
from cubicweb.server.session import hooks_control

from cubes.medicalexp.config import SUBJECT_CONTAINER, ASSESSMENT_CONTAINER

from cubes.brainomics.importers.helpers import generate_container_relations
from cubes.brainomics.test.unittest_schema import create_subject_container_data


class BrainomicsContainersTC(CubicWebTC):
    """ Test proper behavior with respect to the Subject and Assessment containers. """

    def test_create_subject_container_instance(self):
        """ Container entities are created, according to the schema.
        """
        req = self.request()
        create_subject_container_data(req)

    def test_disjoint_container(self):
        e1, r1 = SUBJECT_CONTAINER.structure(self.vreg.schema)
        e2, r2 = ASSESSMENT_CONTAINER.structure(self.vreg.schema)
        self.assertEqual(e1.intersection(e2), set())
        self.assertEqual(r1.intersection(r2), set())

    def test_static_structure_subject(self):
        cfg = SUBJECT_CONTAINER
        self.assertEqual((frozenset(['related_diagnostics', 'related_therapies',
                                     'related_samples', 'related_biopsy',
                                     'taken_in_therapy', 'evaluation_for',
                                     'admission_of', 'step_of', 'survival_of',
                                     'report_on']),
                          frozenset(['Diagnostic', 'Therapy', 'DrugTake',
                                     'Biopsy', 'BiopsySample', 'BloodSample', 'CellCulture',
                                     'Admission', 'AdmissionStep',
                                     'SurvivalData', 'TherapyEvaluation',
                                     'AnatomicPathologyReport'])),
                         cfg.structure(self.vreg.schema))

    def test_static_structure_assessment(self):
        cfg = ASSESSMENT_CONTAINER
        self.assertEqual(
            (frozenset(['results_files', 'configuration_files', 'uses', 'generates',
                        'measure', 'file_entries', 'assessment', 'questionnaire_run',
                        'related_measure', 'measure', 'has_data', 'comments']),
             frozenset(['GenericMeasure', 'ExternalFile', 'FileSet', 'ScoreValue', 'File',
                        'QuestionnaireRun', 'Scan',
                        'GenomicMeasure', 'ColumnRef', 'Answer', 'IHCMeasure', 'IHCResult',
                        'CghResult', 'Mutation', 'PETData', 'MRIData', 'DMRIData', 'Comment'])),
            cfg.structure(self.vreg.schema))

    def test_generate_container_relations(self):
        session = self.session
        with hooks_control(session, session.HOOKS_ALLOW_ALL, 'container'):
            create_subject_container_data(session)
        rset = session.execute('Any X,Y WHERE X of_subject Y')
        self.assertEqual(len(rset), 0)
        rset = session.execute('Any X,Y WHERE X in_assessment Y')
        self.assertEqual(len(rset), 0)
        generate_container_relations(session)
        session = self.session
        rset = session.execute('Any X,Y WHERE X of_subject Y')
        self.assertEqual(len(rset), 4)
        rset = session.execute('Any X,Y WHERE X in_assessment Y')
        self.assertEqual(len(rset), 3)


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-brainomics views/forms/actions/components for web ui"""
from itertools import chain

from cubicweb.web.views.startup import IndexView


class BrainomicsIndexView(IndexView):

    def _display_panel(self, title, content, klass=u''):
        w = self.w
        w(u'<div class="panel %s">' % klass)
        w(u'<div class="panel-heading">')
        w(u'<h3 class="panel-title">%s</h3>' % title)
        w(u'</div>')
        w(u'<div class="panel-body">%s</div>' % content)
        w(u'</div>')

    def display_list_panel(self, items, title, content):
        html = []
        html.append(u'<p>%s</p>' % content)
        html.append(u'<ul class="list-unstyled">')
        for etype, rql, etitle in items:
            if rql is None:
                rql = 'WHERE X is %s' % etype
            rset = self._cw.execute('Any COUNT(X) %s' % rql)
            if not rset[0][0]:
                continue
            rset = self._cw.execute('Any X LIMIT 1 %s' % rql)
            view = self._cw.vreg['views'].select_or_none('outofcontext', self._cw, rset=rset)
            url = self._cw.build_url(rql='Any X %s' % rql, vid='sameetypelist')
            if etitle is None:
                etitle = self._cw._(etype)
            icon = (('<span class="glyphicon glyphicon-%s"></span>' % view.icon)
                    if hasattr(view, 'icon') and view.icon else u'')
            html.append(u'<li><a href="%s">%s %s</a></li>' % (url, icon, etitle))
        html.append(u'</ul>')
        self._display_panel(title, '\n'.join(html), 'panel-info')

    def display_infos(self):
        w = self.w
        rset = self._cw.execute('Any X WHERE X is Card, X wikiid %(t)s', {'t': 'index'})
        if not rset:
            return
        w(u'<div class="row"><div class="col-md-8">')
        w(u'<div class="jumbotron"><p>%s</p></div>'
          % rset.get_entity(0, 0).printable_value('content'))
        w(u'</div><div class="col-md-4">')
        if 'BIGLOGO' in self._cw.uiprops:
            w(u'<img id="biglogo" src="%s"/>' % self._cw.uiprops['BIGLOGO'])
        w(u'</div></div>')

    def display_dashboards(self):
        rset = self._cw.execute('Any X WHERE X is Card, X wikiid %(t)s', {'t': 'dashboards'})
        if not rset:
            return
        entity = rset.get_entity(0, 0)
        self._display_panel(entity.title, entity.printable_value('content'), 'panel-warning')

    def display_examples(self):
        rset = self._cw.execute('Any X WHERE X is Card, X wikiid %(t)s', {'t': 'rqlexamples'})
        if not rset:
            return
        entity = rset.get_entity(0, 0)
        self._display_panel(entity.title, entity.printable_value('content'), 'panel-warning')

    def get_clinical_data_etypes(self):
        measures = set([(r.type, None, None) for r in self._cw.vreg.schema.rschema('concerns').subjects()])
        return chain(((etype, None, None) for etype in ('Study', 'Subject', 'Therapy',
                      'Diagnostic', 'DrugTake')), measures)

    def get_thesaurus_data_etypes(self):
        return ((etype, None, None) for etype in ('Disease', 'BodyLocation', 'MedicalTechnique', 'Drug'))

    def call(self, **kwargs):
        w = self.w
        # Presentation
        w(u'<div class="page-header"><h1>%s</h1></div>' % self._cw.property_value('ui.site-title'))
        self.display_infos()
        # Panels
        w(u'<div class="col-md-12">')
        self.display_dashboards()
        w(u'</div>')
        # Patients
        w(u'<div class="col-md-6">')
        self.display_list_panel(self.get_clinical_data_etypes(),
                                self._cw._(u'Patient directory-based search'),
                                self._cw._('Access patient-related data'))
        w(u'</div>')
        # Vocabulaire
        w(u'<div class="col-md-6">')
        self.display_list_panel(self.get_thesaurus_data_etypes(),
                                self._cw._('Thesaurus-based search'),
                                self._cw._('Access thesaurus-based data for '
                                           'transversal retrieval'))
        w(u'</div>')
        # Examples
        self.display_examples()


def registration_callback(vreg):
    vreg.register_and_replace(BrainomicsIndexView, IndexView)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-suivimp views/forms/actions/components for web ui"""
from logilab.mtconverter import xml_escape
from logilab.common.decorators import monkeypatch

from cubicweb.schema import META_RTYPES, VIRTUAL_RTYPES
from cubicweb.selectors import is_instance
from cubicweb.web.views.primary import PrimaryView

from cubes.brainomics.entities import MEASURES

from cubicweb import NoSelectableObject


###############################################################################
### BRAINOMICS PRIMARY VIEW ###################################################
###############################################################################
class BrainomicsPrimaryView(PrimaryView):
    """ Redefine primary view for html customization """

    def render_entity_attributes(self, entity):
        """Renders all attributes and relations in the 'attributes' section.
        """
        display_attributes = []
        for rschema, _, role, dispctrl in self._section_def(entity, 'attributes'):
            vid = dispctrl.get('vid', 'reledit')
            if rschema.final or vid == 'reledit' or dispctrl.get('rtypevid'):
                value = entity.view(vid, rtype=rschema.type, role=role,
                                    initargs={'dispctrl': dispctrl})
            else:
                rset = self._relation_rset(entity, rschema, role, dispctrl)
                if rset:
                    value = self._cw.view(vid, rset)
                else:
                    value = None
            # pylint: disable=E1101
            label = self._rel_label(entity, rschema, role, dispctrl)
            if value is not None and value != '':
                display_attributes.append((dispctrl.get('order', 9999), label, value))
        # BRAINOMICS MODIFICATIONS - Add specific display rset
        for order, label, value in self.iterate_display_rset(entity, 'attributes'):
            display_attributes.append((order, label, value))
        if display_attributes:
            self.w(u'<dl class="dl-horizontal">')
            for order, label, value in sorted(display_attributes, key=lambda x: x[0]):
                self.render_attribute(label, value)
            self.w(u'</dl>')

    def iterate_display_rset(self, entity, section):
        """ Iterate over the display rset from uicfg """
        # Add specific uicfg rset views
        eschema = entity.e_schema
        if section == 'attributes':
            rsetdefs = self.display_ctrl._attrrsetdefs.get(eschema, [])
        elif section == 'relations':
            rsetdefs = self.display_ctrl._relrsetdefs.get(eschema, [])
        else:
            self.warning('Bad section %s for uicfg display rsets' % section)
            return
        for dispctrl in rsetdefs:
            label = dispctrl.get('label')
            if label:
                label = self._cw._(label)
            else:
                label = u''
            # Get order
            order = dispctrl['order'] if 'order' in dispctrl else 9999
            # Get value
            if 'rql' in dispctrl:
                rset = self._cw.execute(dispctrl['rql'] + ', X eid %(e)s', {'e': entity.eid})
                if rset:
                    value = self._cw.view(dispctrl['vid'], rset=rset)
                    if value:
                        yield order, label, value
            elif 'callback' in dispctrl:
                value = dispctrl.get('callback')(entity)
                if value:
                    yield order, label, value

    def render_entity_relations(self, entity):
        """Renders all relations in the 'relations' section."""
        defaultlimit = self._cw.property_value('navigation.related-limit')
        display_relations = []
        for rschema, tschemas, role, dispctrl in self._section_def(entity, 'relations'):
            if rschema.final or dispctrl.get('rtypevid'):
                vid = dispctrl.get('vid', 'reledit')
                try:
                    rview = self._cw.vreg['views'].select(
                        vid, self._cw, rset=entity.cw_rset, row=entity.cw_row,
                        col=entity.cw_col, dispctrl=dispctrl,
                        rtype=rschema, role=role)
                except NoSelectableObject:
                    continue
                value = rview.render(row=entity.cw_row, col=entity.cw_col,
                                     rtype=rschema.type, role=role)
            else:
                vid = dispctrl.get('vid', 'autolimited')
                limit = defaultlimit if vid == 'autolimited' else None
                rset = self._relation_rset(entity, rschema, role, dispctrl, limit=limit)
                if not rset:
                    continue
                if hasattr(self, '_render_relation'):
                    # pylint: disable=E1101
                    self._render_relation(dispctrl, rset, 'autolimited')
                    warn('[3.9] _render_relation prototype has changed and has '
                         'been renamed to render_relation, please update %s'
                         % self.__class__, DeprecationWarning)
                    continue
                try:
                    rview = self._cw.vreg['views'].select(
                        vid, self._cw, rset=rset, dispctrl=dispctrl)
                except NoSelectableObject:
                    continue
                value = rview.render()
            label = self._rel_label(entity, rschema, role, dispctrl)
            display_relations.append((dispctrl.get('order', 9999), label, value))
        # BRAINOMICS MODIFICATIONS - Add specific display rset
        for order, label, value in self.iterate_display_rset(entity, 'relations'):
            display_relations.append((order, label, value))
        if display_relations:
            for order, label, value in sorted(display_relations, key=lambda x: x[0]):
                self.render_relation(label, value)

    def render_attribute(self, label, value):
        self.w(u'<dt>%(label)s</dt><dd>%(value)s</dd>'
               % {'label': xml_escape(self._cw._(label)),
                  'value': value})

    def render_relation(self, label, value):
        self.w(u'<div class="col-md-12">')
        if label:
            self.w(u'<h4>%s</h4>' % label)
        self.w(value)
        self.w(u'</div>')

    def render_entity(self, entity=None):
        entity = entity if entity else self.cw_rset.get_entity(0,0)
        w = self.w
        # Toolbar
        boxes = list(self._cw.vreg['ctxcomponents'].poss_visible_objects(
            self._cw, rset=self.cw_rset, context='nav-toolbar'))
        if boxes:
            w(u'<div class="row navtoolbar">')
            for box in boxes:
                box.render(w=w)
            w(u'</div>')
        # define rsection for similar behavior than CW's primary view
        uicfg_reg = self._cw.vreg['uicfg']
        if self.rsection is None:
            self.rsection = uicfg_reg.select('primaryview_section',
                                             self._cw, entity=entity)
        if self.display_ctrl is None:
            self.display_ctrl = uicfg_reg.select('primaryview_display_ctrl',
                                                 self._cw, entity=entity)
        # BRAINOMICS MODIFICATIONS - Customize html
        w(u'<div class="tab-content">')
        w(u'<div class="tab-pane active" id="entity-mainview">')
        # Attributes
        w(u'<div class="well">')
        w(u'<h2>%s</h2>' % xml_escape(entity.dc_title()))
        self.render_entity_attributes(entity)
        w(u'</div>')
        # Relations
        w(u'<div class="row">')
        self.render_entity_relations(entity)
        w(u'</div>')
        w(u'</div>')
        # BRAINOMICS MODIFICATIONS - Add tabs
        if 'ctx-entity-tabs' in self._cw.vreg:
            tabs = self._cw.vreg['ctx-entity-tabs'].possible_objects(self._cw, rset=self.cw_rset)
            for tab in tabs:
                tab.render(self.w)
        w(u'</div>')


###############################################################################
### REGISTRATION CALLBACK #####################################################
###############################################################################
def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, (BrainomicsPrimaryView,))
    vreg.register_and_replace(BrainomicsPrimaryView, PrimaryView)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
import warnings

from cubicweb.web import facet
from cubicweb.predicates import is_instance
try:
    from cubicweb.web.facet import AbstractRangeRQLPathFacet
    warnings.warn('AbstractRangeRQLPathFacet are now available. '
                  'The file cubes.brainomics.views.rqlpathfacets should be removed')
except ImportError:
    from cubes.brainomics.views.rqlpathfacets import AbstractRangeRQLPathFacet
    warnings.warn('AbstractRangeRQLPathFacet are backport from CW 3.19')


_ = unicode


def translatable(value):
    @property
    def translate(self, value=value):
        return self._cw._(value).capitalize()
    return translate


class SubjectAgeFacet(AbstractRangeRQLPathFacet, facet.RangeFacet):
    __regid__ = 'subject-age-facet'
    __select__ = facet.RQLPathFacet.__select__ & is_instance('Subject')
    path = ['A admission_of X', 'A subject_age V']
    order = 3
    filter_variable = 'V'
    title = translatable(_('Age'))


class MeasureAgeFacet(facet.RQLPathFacet):
    __regid__ = 'measure-age-facet'
    __select__ = is_instance('Scan', 'QuestionnaireRun', 'GenomicMeasure')
    path = ['S generates X', 'S age_of_subject A']
    order = 1
    filter_variable = 'A'
    title = translatable(_("Subject Age"))


class MeasureHandednessFacet(facet.RQLPathFacet):
    __regid__ = 'measure-handedness-facet'
    __select__ = is_instance('Scan', 'QuestionnaireRun', 'GenomicMeasure')
    path = ['X concerns S', 'S handedness H']
    order = 2
    filter_variable = 'H'
    title = translatable(_('Subject Handedness'))


class MeasureGenderFacet(facet.RQLPathFacet):
    __regid__ = 'measure-gender-facet'
    __select__ = is_instance('Scan', 'QuestionnaireRun', 'GenomicMeasure')
    path = ['X concerns S', 'S gender G']
    order = 3
    filter_variable = 'G'
    title = translatable(_('Subject Gender'))


class BiopsyTissueTypeFacet(facet.AttributeFacet):
    __regid__ = 'biopsy-tissuetype-facet'
    __select__ = facet.AttributeFacet.__select__ & is_instance('Biopsy')
    rtype = 'tissue_type'
    title = translatable(_('tissue_type'))


class BiopsyDatetimeFacet(facet.DateRangeFacet):
    __regid__ = 'biopsy-datetime'
    __select__ = facet.DateRangeFacet.__select__ & is_instance('Biopsy')
    rtype = 'biopsy_date'
    title = translatable(_('biopsy_date'))


class BiopsyDiagnosticLocation(facet.RQLPathFacet):
    __regid__ = 'biopsy-diagnostic-location'
    __select__ = facet.RQLPathFacet.__select__ & is_instance('Biopsy')
    path = ['S based_on X', 'S diagnostic_location L', 'L name N']
    filter_variable = 'N'
    order = 3
    title = translatable(_('diagnostic_location'))


class BiopsyDiagnosticDisease(facet.RQLPathFacet):
    __regid__ = 'biopsy-diagnostic-disease'
    __select__ = facet.RQLPathFacet.__select__ & is_instance('Biopsy')
    path = ['S based_on X', 'S diagnosed_disease L', 'L name N']
    filter_variable = 'N'
    order = 2
    title = translatable(_('diagnosed_disease'))


class BiopsyDiagnosticTechniqueType(facet.RQLPathFacet):
    __regid__ = 'biopsy-diagnostic-technique'
    __select__ = facet.RQLPathFacet.__select__ & is_instance('Biopsy')
    path = ['S based_on X', 'S technique_type L', 'L name N']
    filter_variable = 'N'
    title = translatable(_('TechniqueType'))


class BiopsySampleCellularityAgeFacet(AbstractRangeRQLPathFacet, facet.RangeFacet):
    __regid__ = 'biopsysample-cellularity-facet'
    __select__ = facet.RQLPathFacet.__select__ & is_instance('Biopsy')
    path = ['B extracted_from X', 'B tumoral_cell_percent T']
    order = 1
    filter_variable = 'T'
    title = translatable(_('tumoral_cell_percent'))


class GenomicMeasureType(facet.AttributeFacet):
    __regid__ = 'genomic-measure-type'
    __select__ = facet.AttributeFacet.__select__ & is_instance('GenomicMeasure')
    rtype = 'type'
    order = 1


class BodyLocationBiospyFacet(facet.HasRelationFacet):
    __regid__ = 'bodylocation-biopsy-facet'
    __select__ = facet.HasRelationFacet.__select__ & is_instance('BodyLocation')
    rtype = 'location_on_body'
    role = 'object'
    title = translatable(_('location_on_body'))


class TechniqueTypeBiospyFacet(facet.HasRelationFacet):
    __regid__ = 'techniquetype-biopsy-facet'
    __select__ = facet.HasRelationFacet.__select__ & is_instance('TechniqueType')
    rtype = 'technique_type'
    role = 'object'
    title = translatable(_('technique_type'))


class DiseaseBiospyFacet(facet.HasRelationFacet):
    __regid__ = 'disease-biopsy-facet'
    __select__ = facet.HasRelationFacet.__select__ & is_instance('Disease')
    rtype = 'lesion'
    role = 'object'
    title = translatable(_('lesion'))



# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

""" Schema test """

from cubicweb.devtools.testlib import CubicWebTC
from cubicweb.web.views import uicfg

from cubes.brainomics.views.primary import BrainomicsPrimaryView


class BrainomicsUICFGTC(CubicWebTC):
    """ Test proper uicfg behavior, see https://www.cubicweb.org/ticket/3489097 """

    def setup_database(self):
        """ Several entities involving composite relations are created,
            according to the schema.
        """
        # Clean uicfg for test
        uicfg.primaryview_display_ctrl._relrsetdefs = {}
        uicfg.primaryview_display_ctrl._attrrsetdefs = {}
        # Add entities
        req = self.request()
        t_study = req.create_entity('Study', name=u'Test study',
                                    data_filepath=u'test/data/filepath')
        t_card = req.create_entity('Card', title=u'This is a card.',
                                   reverse_wiki=t_study)
        t_subj = req.create_entity('Subject', identifier=u'test_subj',
                                   gender=u'unknown', handedness=u'mixed')
        t_center = req.create_entity('Center', identifier=u'center_id',
                                     name=u'Test center name')
        t_assessment = req.create_entity('Assessment',
                                         related_study=t_study,
                                         reverse_holds=t_center,
                                         reverse_concerned_by=t_subj)

    def get_display_rset(self, req, entity, type='relations'):
        pview = BrainomicsPrimaryView(req)
        uicfg_reg = req.vreg['uicfg']
        pview.display_ctrl = uicfg_reg.select('primaryview_display_ctrl',
                                              req, entity=entity)
        return list(pview.iterate_display_rset(entity, type))

    def add_center_uicfg(self):
        _pvdc = uicfg.primaryview_display_ctrl
        _pvdc.display_rset('Center', 'relations',
                           {'rql': 'Any S WHERE S concerned_by Y, X holds Y',
                            'vid': 'table',
                            'order': 1,
                            'label': _('Subjects')})

    def test_uihelper_setdefs(self):
        # Add uicfg display_rset
        _pvdc = uicfg.primaryview_display_ctrl
        self.add_center_uicfg()
        _pvdc.display_rset('Gene', 'relations',
                           {'rql': 'Any CGH WHERE CGH genomic_region GR, GR genes X',
                            'vid': 'region-genmeas-table-view',
                            'order': 1,
                            'label': _('CGH results')})
        self.assertEqual(uicfg.primaryview_display_ctrl._attrrsetdefs, {})
        self.assertEqual(uicfg.primaryview_display_ctrl._relrsetdefs['Center'],
                         [{'rql': 'Any S WHERE S concerned_by Y, X holds Y',
                           'order': 1, 'vid': 'table', 'label': u'Subjects'}])
        self.assertEqual(uicfg.primaryview_display_ctrl._relrsetdefs['Gene'],
                         [{'rql': 'Any CGH WHERE CGH genomic_region GR, GR genes X',
                           'vid': 'region-genmeas-table-view',
                           'order': 1,
                           'label': _('CGH results')}])

    def test_uihelper_iterate_rset_relations(self):
        # Add uicfg display_rset
        self.add_center_uicfg()
        req = self.request()
        entity = req.find_one_entity('Center', identifier=u'center_id')
        display_rset = self.get_display_rset(req, entity)
        self.assertEqual(len(display_rset), 1)
        self.assertEqual(display_rset[0][0], 1)
        self.assertEqual(display_rset[0][1], u'Subjects')
        self.assertTrue(display_rset[0][2].startswith('<div id'))

    def test_uihelper_iterate_rset_attributes(self):
        # Add uicfg display_rset
        _pvdc = uicfg.primaryview_display_ctrl
        _pvdc.display_rset('Center', 'attributes',
                           {'callback': lambda x:x.identifier,
                            'order': 2,
                            'label': _('Test callback')})
        self.add_center_uicfg()
        req = self.request()
        entity = req.find_one_entity('Center', identifier=u'center_id')
        display_rset = self.get_display_rset(req, entity, 'attributes')
        self.assertEqual(len(display_rset), 1)
        self.assertEqual(display_rset, [(2, u'Test callback', u'center_id')])

    def test_uihelper_iterate_rset_callback(self):
        self.add_center_uicfg()
        req = self.request()
        _pvdc = uicfg.primaryview_display_ctrl
        _pvdc.display_rset('Center', 'relations',
                           {'callback': lambda x:x.identifier,
                            'order': 2,
                            'label': _('Test callback')})
        entity = req.find_one_entity('Center', identifier=u'center_id')
        display_rset = self.get_display_rset(req, entity)
        self.assertEqual(len(display_rset), 2)
        self.assertEqual(display_rset[1], (2, u'Test callback', u'center_id'))

    def test_uihelper_order(self):
        req = self.request()
        _pvdc = uicfg.primaryview_display_ctrl
        _pvdc.display_rset('Center', 'relations',
                           {'rql': 'Any S WHERE S concerned_by Y, X holds Y',
                            'vid': 'table',
                            'order': 2,
                            'label': _('Subjects 2')})
        _pvdc.display_rset('Center', 'relations',
                           {'rql': 'Any S WHERE S concerned_by Y, X holds Y',
                            'vid': 'table',
                            'order': 0,
                            'label': _('Subjects 0')})
        _pvdc.display_rset('Center', 'relations',
                           {'callback': lambda x:x.identifier,
                            'order': 1,
                            'label': _('Subjects 1')})
        entity = req.find_one_entity('Center', identifier=u'center_id')
        pview = BrainomicsPrimaryView(req)
        html = []
        html_rel = []
        pview.w = html.append
        pview.render_relation = lambda x,y: html_rel.append((x,y))
        pview.render_entity(entity)
        self.assertEqual(html_rel[0][0], 'Subjects 0')
        self.assertEqual(html_rel[1][0], 'Subjects 1')
        self.assertEqual(html_rel[2][0], 'Subjects 2')


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-brainomics entity's classes"""
import os.path as osp

from cubicweb.view import EntityAdapter
from cubicweb.predicates import is_instance

from cubes.medicalexp.entities import ScoreValue


MEASURES = ['GenericMeasure', 'Scan', 'GenericTestRun', 'QuestionnaireRun', 'GenomicMeasure']


class AssessmentICalendarableAdapter(EntityAdapter):
    __regid__ = 'ICalendarable'
    __select__ = EntityAdapter.__select__ & is_instance('Assessment')

    @property
    def start(self):
        return self.entity.datetime

    @property
    def end(self):
        return self.entity.datetime


class DiagnosticICalendarableAdapter(EntityAdapter):
    __regid__ = 'ICalendarable'
    __select__ = EntityAdapter.__select__ & is_instance('Diagnostic')

    @property
    def start(self):
        return self.entity.diagnostic_date


class ScoreValue(ScoreValue):

    @property
    def subject(self):
        if self.reverse_related_infos:
            return self.reverse_related_infos[0]
        if self.measure:
            return self.measure[0].concerns[0]
        if self.score_of_sample:
            return self.score_of_sample[0].reverse_related_samples[0]



# -*- coding: utf-8 -*-
# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
from random import randrange
from optparse import OptionParser
from datetime import timedelta, datetime

import numpy.random as nr

from cubicweb.dataimport import SQLGenObjectStore

from cubes.brainomics.importers.helpers import (get_image_info, import_genes,
                                                import_chromosomes, import_snps)

#http://www.housemd-guide.com/characters/houserules.php
DR_HOUSE_QUOTES = (u"We treat it. If she[he] gets better we know that we're right.",
                   u"Patients always want proof, we're not making cars here, we don't give guarantees.",
                   u"Pretty much all the drugs I prescribe are addictive and dangerous.",
                   u"Patients sometimes get better. You have no idea why, but unless you give a reason they won't pay you. Anybody notice if there's a full moon? ... let's rule out the lunar god and go from there.",
                   u"Occam's Razor.  The simplest explanation is almost always somebody screwed up.",
                   u"I take risks, sometimes patients die. But not taking risks causes more patients to die, so I guess my biggest problem is I've been cursed with the ability to do the math.",
                   u"Idiopathic, from the Latin meaning we're idiots cause we can't figure out what's causing it.",
                   u"If he gets better, I'm right, if he dies, you're right.",
                   u"The eyes can mislead, the smile can lie, but the shoes always tell the truth.",
                   u"Hang up a shingle and condemn the narrowness and greed of Western medicine, you'd make a damn fine living.",
                   u"It is the nature of medicine that you are going to screw up.",
                   u"Right and wrong do exist. Just because you don't know what the right answer is — maybe there's even no way you could know what the right answer is — doesn't make your answer right or even okay. It's much simpler than that. It's just plain wrong.",
                   u"You know what's worse than useless? Useless and oblivious.",
                   u"It is in the nature of medicine that you are gonna screw up. You are gonna kill someone. If you can't handle that reality, pick another profession. Or finish medical school and teach.",
                   u"If it works, we're right. If he dies, it was something else.",
                   u"If her DNA was off by one percentage point she'd be a dolphin.",)

DR_HOUSE_QUOTES_CONCLUSIONS = (u"Never met a diagnostic study I couldn't refute.",
                               u"Never trust doctors.",
                               u"That's a catchy diagnosis, you could dance to that.",
                               u"Tragedies happen.",
                               u"Weird works for me.",
                               u"In case I'm wrong. It has happened.",
                               u"It does tell us something. Though I have no idea what.",
                               u"I solved the case, my work is done.",
                               u"Tests take time. Treatment's quicker.",
                               u"Welcome to the end of the thought process.",
                               u"Sometimes we can't see why normal isn't normal.",
                               u"You want to make things right? Too bad. Nothing's ever right.",
                               u"New is good. Because old ended in death.")


###############################################################################
### DATA GENERATION CONFIGURATION #############################################
###############################################################################
NB_SUBJECTS = 1000
NB_SUBJECTS_GROUPS = 5
AGE_DATETIMES = (datetime.strptime('1/1/1930 1:30 PM', '%m/%d/%Y %I:%M %p'),
                 datetime.strptime('1/1/1990 4:50 AM', '%m/%d/%Y %I:%M %p'))
PROJECT_DATETIMES = (datetime.strptime('1/1/2010 1:30 PM', '%m/%d/%Y %I:%M %p'),
                     datetime.strptime('1/1/2013 4:50 AM', '%m/%d/%Y %I:%M %p'))
SEX = {0: u'male', 1: u'female'}
HANDEDNESS = {0: u'right', 1: u'left', 2: u'ambidextrous', 3: u'mixed'}


###############################################################################
### UTILITY FUNCTIONS #########################################################
###############################################################################
def random_date(_type=None, start=None, end=None):
    """
    This function will return a random datetime between two datetime
    objects.
    From http://stackoverflow.com/questions/553303/generate-a-random-date-between-two-other-dates
    """
    if start:
        end = PROJECT_DATETIMES[1]
    elif _type=='age':
        start, end = AGE_DATETIMES[0], AGE_DATETIMES[1]
    else:
        start, end = PROJECT_DATETIMES[0], PROJECT_DATETIMES[1]
    delta = end - start
    int_delta = (delta.days * 24 * 60 * 60) + delta.seconds
    random_second = randrange(int_delta)
    return (start + timedelta(seconds=random_second))

def create_protocol(store, study, identifier):
    """ Create a protocol """
    date = random_date()
    protocol_eid = store.create_entity('Protocol', identifier=identifier,
                                       start_datetime=date).eid
    store.relate(protocol_eid, 'related_study', study, subjtype='Protocol')
    return protocol_eid

def import_questionnaire(store, subject_eid, center_eid, study_eid,
                         questionnaire_eid, questions, label, protocol_eid):
    date = random_date()
    assessment = store.create_entity('Assessment', datetime=date)
    store.relate(assessment.eid, 'protocols', protocol_eid, subjtype='Assessment')
    store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    user_ident = nr.randint(10)
    user_ident = u'parent' if user_ident>8 else u'subject'
    q_qrun = store.create_entity('QuestionnaireRun',
                                   user_ident=user_ident, datetime=date,
                                   iteration=1, completed=True, valid=True,
                                   instance_of=questionnaire_eid)
    store.relate(q_qrun.eid, 'related_study', study_eid, subjtype='QuestionnaireRun')
    store.relate(q_qrun.eid, 'concerns', subject_eid, subjtype='QuestionnaireRun')
    store.relate(assessment.eid, 'generates', q_qrun.eid, subjtype='Assessment')
    for name, (question, _min, _max) in questions.iteritems():
        score_value = store.create_entity('Answer', value=nr.randint(_min, _max),
                                          question=question, datetime=date,
                                          questionnaire_run=q_qrun.eid)

def import_genomic(store, subject_eid, center_eid, study_eid, platform, type, protocol_eid):
    date = random_date()
    assessment = store.create_entity('Assessment', datetime=date)
    store.relate(assessment.eid, 'protocols', protocol_eid, subjtype='Assessment')
    store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    measure = store.create_entity('GenomicMeasure', type=type,
                                  platform=platform if platform else None,
                                  completed=True, valid=True, related_study=study_eid)
    _file = store.create_entity('ExternalFile', name='genomic_%s' % subject_eid,
                                absolute_path=False, filepath=u'%s/genomics.tar.gz' % subject_eid)
    store.relate(measure.eid, 'results_files', _file.eid)
    store.relate(measure.eid, 'concerns', subject_eid, subjtype='GenomicMeasure')
    store.relate(assessment.eid, 'generates', measure.eid, subjtype='Assessment')
    return measure.eid

def create_genomic_region(store):
    """ Add a GenomicRegion """
    start = nr.randint(1000000)
    stop = nr.randint(start, 1000000)
    chromosome = store.rql('Any X ORDERBY RANDOM() LIMIT 1 WHERE X is Chromosome')
    entity = store.create_entity('GenomicRegion', start=start, stop=stop,
                                 width=stop-start,
                                 cytoband_start=u'q+%s' % start,
                                 cytoband_stop=u'q+%s' % stop,
                                 chromosome=chromosome[0][0])
    for gene in store.rql('Any X ORDERBY RANDOM() LIMIT 100 WHERE X is Gene, '
                          'X chromosomes Y, Y eid %(e)s', {'e': chromosome[0][0]}):
        store.relate(entity.eid, 'genes', gene[0], subjtype='GenomicRegion')
    return entity.eid

def import_cgh_results(store, subject_eid, center_eid, study_eid, platform, protocol_eid):
    """ Add CGH results """
    measure = import_genomic(store, subject_eid, center_eid, study_eid,
                             platform, u'CGH', protocol_eid)
    for region in range(nr.randint(100)):
        region = create_genomic_region(store)
        store.create_entity('CghResult', cgh_ratio=nr.random(), log2_ratio=nr.random(),
                            status=(u'L', u'G')[nr.randint(1)],
                            numprobes=nr.randint(100),
                            related_measure=measure, genomic_region=region)

def import_sequencing_results(store, subject_eid, center_eid, study_eid, platform, protocol_eid):
    """ Add CGH results """
    measure = import_genomic(store, subject_eid, center_eid, study_eid,
                             platform, u'Sequencing', protocol_eid)
    for mutation in range(nr.randint(100)):
        gene = store.rql('Any X ORDERBY RANDOM() LIMIT 1 WHERE X is Gene')[0][0]
        store.create_entity('Mutation', position_in_gene=nr.randint(1000),
                            mutation_type=u'SNP', ploidy=(u'Het', u'Hom')[nr.randint(1)],
                            biological_classification=u'Substitution',
                            related_measure=measure,
                            related_gene=gene,
                            reference_base=(u'A', u'T', u'G', u'C')[nr.randint(3)],
                            variant_base=(u'A', u'T', u'G', u'C')[nr.randint(3)],
                            variant_frequency=100*nr.random())

def import_anat_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2, protocol_eid):
    date = random_date()
    assessment = store.create_entity('Assessment', datetime=date)
    store.relate(assessment.eid, 'protocols', protocol_eid, subjtype='Assessment')
    store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    device = nr.randint(1)
    mri_data = store.create_entity('MRIData',
                                   shape_x=128, shape_y=128, shape_z=64,
                                   sequence=u'T1',
                                   voxel_res_x=1.5, voxel_res_y=1.5, voxel_res_z=1.5).eid
    scan_data = store.create_entity('Scan', has_data=mri_data,
                                    related_study=study_eid,
                                    label=u'anatomy', type=u'normalized T1',
                                    format=u'nii.gz',
                                    uses_device=mri_1 if device else mri_2,
                                    completed=True, valid=True).eid
    _file = store.create_entity('ExternalFile', name='anat_%s' % subject_eid,
                                absolute_path=False, filepath=u'%s/anat.nii.gz' % subject_eid)
    store.relate(scan_data, 'results_files', _file.eid)
    store.relate(scan_data, 'concerns', subject_eid, subjtype='Scan')
    store.relate(assessment.eid, 'generates', scan_data, subjtype='Assessment')

def import_fmri_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2, protocol_eid):
    date = random_date()
    assessment = store.create_entity('Assessment', datetime=date)
    store.relate(assessment.eid, 'protocols', protocol_eid, subjtype='Assessment')
    store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    device = nr.randint(1)
    mri_data = store.create_entity('MRIData',
                                   sequence=u'EPI',
                                   shape_x=128, shape_y=128, shape_z=64, shape_t=256,
                                   voxel_res_x=1.5, voxel_res_y=1.5, voxel_res_z=1.5).eid
    scan_data = store.create_entity('Scan', has_data=mri_data,
                                    related_study=study_eid,
                                    label=u'bold', type=u'preprocessed fMRI',
                                    format=u'nii.gz',
                                    uses_device=mri_1 if device else mri_2,
                                    completed=True, valid=True).eid
    _file = store.create_entity('ExternalFile', name='bold_%s' % subject_eid,
                                absolute_path=False, filepath=u'%s/bold.nii.gz' % subject_eid)
    store.relate(scan_data, 'results_files', _file.eid)
    store.relate(scan_data, 'concerns', subject_eid, subjtype='Scan')
    store.relate(assessment.eid, 'generates', scan_data, subjtype='Assessment')

def import_constrat_images(store, subject_eid, center_eid, study_eid, mri_1, mri_2, protocol_eid):
    date = random_date()
    assessment = store.create_entity('Assessment', datetime=date)
    store.relate(assessment.eid, 'protocols', protocol_eid, subjtype='Assessment')
    store.relate(assessment.eid, 'related_study', study_eid, subjtype='Assessment')
    store.relate(center_eid, 'holds', assessment.eid)
    store.relate(subject_eid, 'concerned_by', assessment.eid)
    for ind, label in enumerate((u'Audio-Video', u'Text reading', u'Video reading',
                                 u'Social behavior', u'Audio interaction')):
        for smap in (u'c map', u't map'):
            mri_data = store.create_entity('MRIData',
                                           sequence=u'EPI',
                                           shape_x=128, shape_y=128, shape_z=64,
                                           voxel_res_x=1.5, voxel_res_y=1.5, voxel_res_z=1.5).eid
            scan_data = store.create_entity('Scan', has_data=mri_data,
                                            related_study=study_eid,
                                            label=label, type=smap,
                                            format=u'nii.gz',
                                            completed=True, valid=True).eid
            filepath = '%s/%s_%s.nii.gz' % (smap.replace(' ', ''), subject_eid, ind)
            _file = store.create_entity('ExternalFile', name='scan_%s_%s' % (subject_eid, ind),
                                        absolute_path=False, filepath=filepath)
            store.relate(scan_data, 'results_files', _file.eid)
            store.relate(scan_data, 'concerns', subject_eid, subjtype='Scan')
            store.relate(assessment.eid, 'generates', scan_data, subjtype='Assessment')


###############################################################################
### IMPORT ####################################################################
###############################################################################
if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option("-c", "--chromosomes", dest="chromosomes",
                      help="Json file of chromosomes")
    parser.add_option("-g", "--genes", dest="genes",
                      help="Json file of genes")
    parser.add_option("-s", "--snps", dest="snps",
                      help="Json file of snps")
    (options, args) = parser.parse_args(__args__)

    store = SQLGenObjectStore(session)


    ###############################################################################
    ### GENETICS DATA #############################################################
    ###############################################################################
    platform = None
    if options.chromosomes:
        chrs = import_chromosomes(options.chromosomes)
        chr_map = {}
        for _chr in chrs:
            print 'chr', _chr['name']
            _chr = store.create_entity('Chromosome', **_chr)
            chr_map.setdefault(_chr['name'], _chr.eid)
        store.flush()
        if options.genes:
            genes = import_genes(options.chromosomes, options.genes)
            for gene in genes:
                print 'gene', gene['name'], gene['chromosome']
                chromosome = gene.pop('chromosome')
                chromosome = chr_map.get(chromosome)
                if not chromosome:
                    continue
                gene = store.create_entity('Gene', **gene)
                store.relate(gene.eid, 'chromosomes', chromosome)
            store.flush()
        if options.snps:
            snps = import_snps(options.chromosomes, options.snps)
            snp_eids = []
            for ind, snp in enumerate(snps):
                print 'snp', snp['rs_id']
                snp['chromosome'] = chr_map[snp['chromosome']]
                snp = store.create_entity('Snp', **snp)
                snp_eids.append(snp.eid)
                if ind and ind % 100000 == 0:
                    store.flush()
            store.flush()
        else:
            snp_eids = []
        # Platform
        platform = {'identifier': 'Affymetrix_6.0'}
        platform = store.create_entity('GenomicPlatform', **platform).eid
        for snp_eid in snp_eids:
            store.relate(platform, 'related_snps', snp_eid)
        store.flush()

    ###############################################################################
    ### STUDY/PI/CENTERS ##########################################################
    ###############################################################################
    study = store.create_entity('Study', name=u'Demo study',
                                data_filepath=u'/tmp/demo_data',
                                description=u'Demo study with random data',
                                keywords=u'demo;random;semantic').eid
    pi_1 = store.create_entity('Investigator', identifier=u'pi_1',
                               firstname=u'Angus', lastname=u'Young', title=u'PhD',
                               institution=u'Research Center', department=u'Cognitive Sciences').eid
    pi_2 = store.create_entity('Investigator', identifier=u'pi_2',
                               firstname=u'Malcom', lastname=u'Young', title=u'MD',
                               institution=u'Research Center', department=u'Cognitive Sciences').eid
    pi_3 = store.create_entity('Investigator', identifier=u'pi_3',
                               firstname=u'Brian', lastname=u'Jonhson', title=u'PhD',
                               institution=u'Research Center', department=u'IT').eid
    pi_4 = store.create_entity('Investigator', identifier=u'pi_4',
                               firstname=u'Phil', lastname=u'Rudd', title=u'PhD',
                               institution=u'Research Center', department=u'IT').eid
    center = store.create_entity('Center', identifier=u'center', name=u'Demo center',
                                 department=u'Medical research', city=u'Paris',
                                 country=u'France').eid
    mri_1 = store.create_entity('Device', name=u'Magnetom', model=u'Trio',
                                manufacturer=u'Siemens',
                                serialnum=1234, hosted_by=center).eid
    mri_2 = store.create_entity('Device', name=u'Ingenia', model=u'3.0 T',
                                manufacturer=u'Philips',
                                serialnum=4321, hosted_by=center).eid


    ###############################################################################
    ### SCOREDEFS #################################################################
    ###############################################################################
    score_defs = {}
    possible_values = u'0 = Control; 1 = Autism; 2 = Aspergers; 3 = PDD-NOS; 4 = Aspergers or PDD-NOS'
    score_defs['dsm_iv_tr'] = (store.create_entity('ScoreDefinition', name=u'DSM-IV-TR Diagnostic Category', category=u'biological', type=u'numerical', possible_values=possible_values).eid, 0, 4)
    score_defs['scq_total'] = (store.create_entity('ScoreDefinition', name=u'Social Communication Questionnaire Total', category=u'behavioral', type=u'numerical', possible_values=u'0-39').eid, 0, 39)
    score_defs['aq_total'] = (store.create_entity('ScoreDefinition', name=u'Total Raw Score of the Autism Quotient', category=u'behavioral', type=u'numerical', possible_values=u'0-50').eid, 0, 50)
    score_defs['fiq_WASI'] = (store.create_entity('ScoreDefinition', name=u'FIQ Standard Score - WASI', category=u'FIQ', type=u'numerical').eid, 50, 160)
    score_defs['viq_WASI'] = (store.create_entity('ScoreDefinition', name=u'VIQ Standard Score - WASI', category=u'VIQ', type=u'numerical').eid, 55, 160)
    score_defs['piq_WASI'] = (store.create_entity('ScoreDefinition', name=u'PIQ Standard Score - WASI', category=u'PIQ', type=u'numerical').eid, 53, 160)


    store.flush()


    ###############################################################################
    ### QUESTIONNAIRES ############################################################
    ###############################################################################
    # ADOS
    name = u'Autism Diagnostic Observation Schedule Module (ADOS)'
    ados_questionnaire_eid = store.create_entity('Questionnaire', name=name, language=u'en', version=u'v 0.2.1', identifier=u'ados', type=u'behavioral').eid
    ados_questions = {}
    text = u'Autism Diagnostic Observation Schedule Module'
    ados_questions['ados_module'] = (store.create_entity('Question', identifier=u'ados_module', position=0, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'1-4').eid, 1, 4)
    text = u'Classical Total ADOS Score (Communication subscore + Social Interaction subscore)'
    ados_questions['ados_total'] = (store.create_entity('Question', identifier=u'ados_total', position=1, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-22').eid, 0, 22)
    text = u'Communication Total Subscore of the Classic ADOS'
    ados_questions['ados_comm'] = (store.create_entity('Question', identifier=u'ados_comm', position=2, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-8').eid, 0, 8)
    text = u'Social Total Subscore of the Classic ADOS'
    ados_questions['ados_social'] = (store.create_entity('Question', identifier=u'ados_social', position=3, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-14').eid, 0, 14)
    text = u'Stereotyped Behaviors and Restricted Interests Total Subscore of the Classic ADOS'
    ados_questions['ados_stereo_behav'] = (store.create_entity('Question', identifier=u'ados_stereo_behav', position=4, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-8').eid, 0, 8)
    text = u'Was ADOS scored and administered by research reliable personnel?'
    possible_answers = u'0 = not research reliable; 1 = research reliable'
    ados_questions['ados_rsrch_reliable'] = (store.create_entity('Question', identifier=u'ados_rsrch_reliable', position=5, text=text, type=u'boolean', questionnaire=ados_questionnaire_eid, possible_answers=possible_answers).eid, 0, 1)
    text = u'Social Affect Total Subscore for Gotham Algorithm of the ADOS'
    ados_questions['ados_gotham_soc_affect'] = (store.create_entity('Question', identifier=u'ados_gotham_soc_affect', position=6, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-20').eid, 0, 20)
    text = u'Restrictive and Repetitive Behaviors Total Subscore for Gotham Algorithm of the ADOS'
    ados_questions['ados_gotham_rrb'] = (store.create_entity('Question', identifier=u'ados_gotham_rrb', position=7, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-8').eid, 0, 8)
    text = u'Social Affect Total + Restricted and Repetitive Behaviors Total'
    ados_questions['ados_gotham_total'] = (store.create_entity('Question', identifier=u'ados_gotham_total', position=8, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'0-28').eid, 0, 28)
    text = u'Individually Calibrated Severity Score for Gotham Algorithm of the ADOS'
    ados_questions['ados_gotham_severity'] = (store.create_entity('Question', identifier=u'ados_gotham_severity', position=9, text=text, type=u'numerical', questionnaire=ados_questionnaire_eid, possible_answers=u'1-10').eid, 1, 10)

    # VINELAND
    vineland_questionnaire_eid = store.create_entity('Questionnaire', name=u'Vineland Adaptive Behavior Scales', identifier=u'vineland', language=u'fr', version=u'v 0.1.0', type=u'behavioral').eid
    vineland_questions = {}
    text = u'Vineland Adaptive Behavior Scales Receptive Language V Scaled Score'
    vineland_questions['vineland_receptive_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_receptive_v_scaled', position=0, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Expressive Language V Scaled Score'
    vineland_questions['vineland_expressive_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_expressive_v_scaled', position=1, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Written Language V Scaled Score'
    vineland_questions['vineland_written_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_written_v_scaled', position=2, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Communication Standard Score'
    vineland_questions['vineland_communication_standard'] = (store.create_entity('Question', identifier=u'vineland_communication_standard', position=3, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'20-160').eid, 20, 160)
    text = u'Vineland Adaptive Behavior Scales Personal Daily Living Skills V Scaled Score'
    vineland_questions['vineland_personal_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_personal_v_scaled', position=4, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Domestic Daily Living Skills V Scaled Score'
    vineland_questions['vineland_domestic_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_domestic_v_scaled', position=5, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Community Daily Living Skills V Scaled Score'
    vineland_questions['vineland_community_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_community_v_scaled', position=6, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Daily Living Skills Standard Score'
    vineland_questions['vineland_dailylvng_standard'] = (store.create_entity('Question', identifier=u'vineland_dailylvng_standard', position=7, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'20-160').eid, 20, 160)
    text = u'Vineland Adaptive Behavior Scales Interpersonal Relationships V Scaled Score'
    vineland_questions['vineland_interpersonal_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_interpersonal_v_scaled', position=8, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Play and Leisure Time V Scaled Score'
    vineland_questions['vineland_play_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_play_v_scaled', position=9, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior ScalesCoping Skills V Scaled Score'
    vineland_questions['vineland_coping_v_scaled'] = (store.create_entity('Question', identifier=u'vineland_coping_v_scaled', position=10, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1-24').eid, 1, 24)
    text = u'Vineland Adaptive Behavior Scales Socialization Standard Score'
    vineland_questions['vineland_social_standard'] = (store.create_entity('Question', identifier=u'vineland_social_standard', position=11, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'20-160').eid, 20, 160)
    text = u'Sum of Vineland Standard Scores (Communication + Daily Living Skills + Socialization)'
    vineland_questions['vineland_sum_scores'] = (store.create_entity('Question', identifier=u'vineland_sum_scores', position=12, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'76-480').eid, 76, 480)
    text = u'Vineland Adaptive Behavior Composite Standard score'
    vineland_questions['vineland_abc_standard'] = (store.create_entity('Question', identifier=u'vineland_abc_standard', position=13, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'20-160').eid, 20, 160)
    text = u'Vineland Adaptive Behavior Scales Informant'
    vineland_questions['vineland_informant'] = (store.create_entity('Question', identifier=u'vineland_informant', position=14, text=text, type=u'numerical', questionnaire=vineland_questionnaire_eid, possible_answers=u'1 = parent; 2 = self').eid, 1, 2)

    store.flush()

    ###############################################################################
    ### SUBJECTS ##################################################################
    ###############################################################################
    groups = []
    for ind in range(NB_SUBJECTS_GROUPS):
        groups.append(store.create_entity('SubjectGroup', identifier=u'demo_group_%s' % ind,
                                          name=u'Demo group of subjects %s' % ind).eid)
    subjects = []
    for ind in range(NB_SUBJECTS):
        print ind
        date = random_date('age')
        subject = store.create_entity('Subject', identifier='demo_subject_%s' % ind,
                                      gender=SEX[nr.randint(len(SEX))],
                                      date_of_birth=date,
                                      handedness=HANDEDNESS[nr.randint(len(HANDEDNESS))]).eid
        subjects.append(subject)
        store.relate(subject, 'related_studies', study)
        store.relate(subject, 'related_groups', groups[nr.randint(len(groups))])
        # Scores
        for name, (score, _min, _max) in score_defs.iteritems():
            score_value = store.create_entity('ScoreValue', definition=score,
                                              datetime=random_date(), value=nr.randint(_min, _max)).eid
            store.relate(subject, 'related_infos', score_value)
        # Admissions
        admdate = random_date()
        if nr.randint(3):
            enddate = random_date(start=admdate)
        else:
            enddate = None
        admission = store.create_entity('Admission',
                                        admission_date=admdate,
                                        admission_end_date=enddate,
                                        admission_of=subject,
                                        admission_in=study)
        for i in range(nr.randint(5)):
            admdate = random_date(start=admdate)
            step = store.create_entity('AdmissionStep',
                                       step_date=admdate,
                                       name=u'Step %s' % i,
                                       step_of=admission.eid)
    store.flush()
    store.commit()


    ###############################################################################
    ### QUESTIONNAIRE RUNS ########################################################
    ###############################################################################
    protocol_eid = create_protocol(store, study, u'Demo questionnaire protocol')
    for subject_eid in subjects:
        print subject_eid, 'questionnaire'
        import_questionnaire(store, subject_eid, center, study,
                             ados_questionnaire_eid, ados_questions,
                             'ados', protocol_eid)
        import_questionnaire(store, subject_eid, center, study,
                             vineland_questionnaire_eid, vineland_questions,
                             'vineland', protocol_eid)
    store.flush()
    store.commit()


    ###############################################################################
    ### GENOMIC MEASURES ##########################################################
    ###############################################################################
    protocol_eid = create_protocol(store, study, u'Demo genomics protocol')
    for subject_eid in subjects:
        print subject_eid, 'genomics'
        measure = import_genomic(store, subject_eid, center, study, platform, u'SNP', protocol_eid)

    store.flush()
    store.commit()

    chromosome = store.rql('Any X ORDERBY RANDOM() LIMIT 1 WHERE X is Chromosome')
    if chromosome:
        for subject_eid in subjects:
            # Only few subject
            if nr.randint(10):
                print subject_eid, 'cgh'
                measure = import_cgh_results(store, subject_eid, center, study,
                                             platform, protocol_eid)
        store.flush()
        store.commit()

        for subject_eid in subjects:
            # Only few subject
            if nr.randint(10):
                print subject_eid, 'sequencing'
                measure = import_sequencing_results(store, subject_eid, center, study,
                                                    platform, protocol_eid)
        store.flush()
        store.commit()


    ###############################################################################
    ### IMAGES MEASURES ###########################################################
    ###############################################################################
    anat_eid = create_protocol(store, study, u'Demo anat protocol')
    fmri_eid = create_protocol(store, study, u'Demo fmri protocol')
    cmap_eid = create_protocol(store, study, u'Demo cmap protocol')
    for subject_eid in subjects:
        print subject_eid, 'images'
        import_anat_images(store, subject_eid, center, study, mri_1, mri_2, anat_eid)
        import_fmri_images(store, subject_eid, center, study, mri_1, mri_2, fmri_eid)
        import_constrat_images(store, subject_eid, center, study, mri_1, mri_2, cmap_eid)
    store.flush()
    store.commit()


    ###############################################################################
    ### DISEASE/THERAPY ###########################################################
    ###############################################################################
    diseases = [r[0] for r in store.rql('Any X ORDERBY RANDOM() LIMIT 10 '
                                        'WHERE X is Disease')]
    locations = [r[0] for r in store.rql('Any X ORDERBY RANDOM() LIMIT 20 '
                                         'WHERE X is BodyLocation')]
    techs = [r[0] for r in store.rql('Any X ORDERBY RANDOM() LIMIT 20 '
                                     'WHERE X is MedicalTechnique')]
    drugs = [r[0] for r in store.rql('Any X ORDERBY RANDOM() LIMIT 20 '
                                         'WHERE X is Drug')]
    if diseases:
        for subject_eid in subjects:
            disease = diseases[nr.randint(len(diseases))]
            location = locations[nr.randint(len(locations))]
            tech = techs[nr.randint(len(techs))]
            # Diagnostic
            diag_date = random_date()
            quote = DR_HOUSE_QUOTES_CONCLUSIONS[nr.randint(len(DR_HOUSE_QUOTES_CONCLUSIONS))]
            quote = u'``%s [...]`` [Dr House' % quote[:200]
            diag = store.create_entity('Diagnostic', diagnostic_date=diag_date,
                                       diagnostic_location=location,
                                       diagnosed_disease=disease,
                                       technique_type=tech,
                                       conclusion=quote).eid
            store.relate(subject_eid, 'related_diagnostics', diag)
            measure = store.rql('Any X WHERE X concerns S, S eid %(e)s', {'e': subject_eid})
            if measure:
                store.relate(diag, 'based_on', measure[0][0])
            # Therapy
            drug = drugs[nr.randint(len(drugs))]
            store.relate(subject_eid, 'related_diseases', disease)
            if nr.randint(3):
                store.relate(subject_eid, 'related_lesions', location)
            start_date = random_date(start=diag_date)
            stop_date = random_date(start=start_date)
            therapy = store.create_entity('Therapy',
                                          start_date=start_date,
                                          stop_date=stop_date if nr.randint(2)
                                          else None)
            store.relate(therapy.eid, 'therapy_for', disease)
            store.relate(therapy.eid, 'based_on_diagnostic', diag)
            store.relate(diag, 'leads_to_therapies', therapy.eid)
            store.relate(subject_eid, 'related_therapies', therapy.eid)
            stop_date = random_date(start=start_date)
            drugtake = store.create_entity('DrugTake',
                                           start_taking_date=start_date,
                                           stop_taking_date=stop_date if nr.randint(2)
                                           else None,
                                           taken_in_therapy=therapy.eid,
                                           drug=drug)
            # TechnicalAnalysis
            quote = DR_HOUSE_QUOTES[nr.randint(len(DR_HOUSE_QUOTES))]
            quote = u'``%s [...]`` [Dr House' % quote[:200]
            tech = store.create_entity('TechnicalAnalysis', analysis_date=random_date(diag_date),
                        conclusion=quote,
                        technique_type=techs[nr.randint(len(techs))]).eid
            store.relate(tech, 'performed_in_therapy', therapy.eid)
    store.flush()
    store.commit()




# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from logilab.mtconverter import xml_escape
from logilab.common.registry import objectify_predicate

from cubicweb import tags
from cubicweb.utils import json_dumps
from cubicweb.predicates import (none_rset, one_line_rset, is_instance, nonempty_rset,
                                 has_related_entities, match_view, match_user_groups,
                                 anonymous_user, relation_possible)

from cubicweb.web import component
from cubicweb.web.views import facets
from cubicweb.web.views.boxes import EditBox
from cubicweb.web.views.bookmark import BookmarksBox
from cubicweb.web.views.basecomponents import ApplicationName, HeaderComponent
from cubicweb.web.httpcache import NoHTTPCacheManager

from cubes.bootstrap.views.basecomponents import AUSDropDownBox

from cubes.brainomics.views.download import ALL_DOWNLOADABLE

LINKED_ENTITIES = ('Subject',)

_ = unicode


@objectify_predicate
def has_dynamic_facets(cls, req, rset=None, **kwargs):
    dfreg = req.vreg.get('dynamic-facet')
    if dfreg and list(dfreg.possible_objects(req, rset=rset)):
        return 1
    return 0


class BrainomicsLinksCtx(component.EntityCtxComponent):
    """ Component used to display related informations and links,
    referenced by the registry 'ctx-related-links'.
    """
    context = 'left'
    __regid__ = 'links-ctx'
    order = 1
    __select__ = component.EntityCtxComponent.__select__ & is_instance(*LINKED_ENTITIES)

    def render(self, w, **kwargs):
        links = list(self._cw.vreg['ctx-related-links'].possible_objects(self._cw, rset=self.cw_rset))
        if not links:
            return
        w(u'<div class="well">')
        w(u'<h4>%s</h4>' % self._cw._('Related entities'))
        w(u'<div class="btn-toolbar">')
        # Render links
        for link in links:
            infos = link.related_infos(self.cw_rset)
            if infos:
                w(u'<div class="btn-group-vertical btn-block">')
                w(u'<div class="btn-group">')
                w(u'<a class="btn btn-primary dropdown-toggle" '
                  u'data-toggle="dropdown" href="#">')
                w(u'%s <span class="caret"></span></a>' % link.get_title())
                w(u'<ul class="dropdown-menu">')
                for ahref, title in infos:
                    w(u'<li><a href="%s">%s</a></li>' % (ahref, title))
                w(u'</ul>')
                w(u'</div>')
                w(u'</div>')
        w(u'</div>')
        w(u'</div>')


class BrainomicsLinksDropdown(component.EntityCtxComponent):
    """ Component used to display related informations and links,
    referenced by the registry 'ctx-links'.
    """
    __abstract__ = True
    __registry__ = 'ctx-related-links'

    def get_title(self):
        return

    def related_infos(self, rset):
        return []


class BrainomicsLinksStudies(BrainomicsLinksDropdown):
    """ Component used to display related studies
    """
    __select__ = BrainomicsLinksDropdown.__select__ & is_instance('Subject')
    __regid__ = 'navtoolbar-studies-ctx'

    def get_title(self):
        return self._cw._('Related studies')

    def related_infos(self, rset):
        studies = set()
        for entity in self.cw_rset.entities():
            for study in entity.related_studies:
                studies.add((study.absolute_url(), study.dc_title()))
        return studies


class BrainomicsLinksGroups(BrainomicsLinksDropdown):
    """ Component used to display related groups
    """
    __select__ = BrainomicsLinksDropdown.__select__ & is_instance('Subject')
    __regid__ = 'navtoolbar-groups-ctx'

    def get_title(self):
        return self._cw._('Related groups')

    def related_infos(self, rset):
        groups = set()
        for entity in self.cw_rset.entities():
            for study in entity.related_groups:
                groups.add((study.absolute_url(), study.dc_title()))
        return groups


class BrainomicsLinksCenters(BrainomicsLinksDropdown):
    """ Component used to display related centers
    """
    __select__ = BrainomicsLinksDropdown.__select__ & is_instance('Subject')
    __regid__ = 'navtoolbar-centers-ctx'

    def get_title(self):
        return self._cw._('Related centers')

    def related_infos(self, rset):
        centers = set()
        for entity in self.cw_rset.entities():
            for study in entity.related_centers:
                centers.add((study.absolute_url(), study.dc_title()))
        return centers


class BrainomicsLinksBrowse(BrainomicsLinksDropdown):
    """ Component used to display browser
    """
    __select__ = BrainomicsLinksDropdown.__select__ & is_instance('Subject')
    __regid__ = 'navtoolbar-browse-ctx'

    def get_title(self):
        return self._cw._('Browse')

    def related_infos(self, rset):
        possible_actions = self._cw.vreg['actions'].possible_actions(self._cw, self.cw_rset)
        links = set()
        for link in list(possible_actions.get('browse-links', [])):
            links.add((link.url(), link.title))
        return links


###############################################################################
### ADMIN ACTION/BOX ##########################################################
###############################################################################
class BrainomicsEditBox(EditBox):
    context = 'left'
    order = -10
    __select__ = EditBox.__select__ & match_user_groups('users', 'managers')

    def init_rendering(self):
        pass

    def get_actions(self):
        actions = self._cw.vreg['actions'].possible_actions(self._cw, self.cw_rset)
        all_actions = []
        for category in ('mainactions', 'moreactions', 'action-admin'):
            for action in actions.get(category, ()):
                all_actions.append(action)
        return all_actions

    def render(self, w, **kwargs):
        actions = self.get_actions()
        if actions:
            w(u'<div class="well admintools">')
            w(u'<h4>%s</h4>' % self._cw._('Administration tools'))
            for action in actions:
                if hasattr(action, 'title'):
                    icon = getattr(action, 'icon', 'edit')
                    title = self._cw._(action.title)
                    w(u'<a class="btn btn-primary btn-block" href="%s">' % action.url())
                    w(u'<i class="glyphicon glyphicon-%s"></i> %s</a>' % (icon, title))
            w(u'</div>')


class BrainomicsBookmarksBox(BookmarksBox, HeaderComponent):
    """display a box containing all user's bookmarks"""
    __regid__ = 'bookmarks_box'
    context = _('header-right')
    order = HeaderComponent.order

    def render_action(self, url, title):
        return u'<li><a href="%s" role="menuitem">%s</a></li>' % (url, title)

    def render(self, w):
        self.init_rendering()
        req = self._cw
        ueid = self._cw.user.eid
        html = []
        if self.can_delete:
            req.add_js('cubicweb.ajax.js')
        if self.can_edit:
            # Create bookmark this page link
            linkto = 'bookmarked_by:%s:subject' % ueid
            path = req.relative_path()
            url = req.vreg['etypes'].etype_class('Bookmark').cw_create_url(
                req, __linkto=linkto, path=path)
            html.append(self.render_action(url, req._('bookmark this page')))
            # Edit bookmarks
            if self.bookmarks_rset:
                if req.user.is_in_group('managers'):
                    bookmarksrql = 'Bookmark B WHERE B bookmarked_by U, U eid %s' % ueid
                    erset = self.bookmarks_rset
                else:
                    bookmarksrql = ('Bookmark B WHERE B bookmarked_by U, '
                                    'B owned_by U, U eid %(x)s')
                    erset = req.execute(bookmarksrql, {'x': ueid},
                                        build_descr=False)
                    bookmarksrql %= {'x': ueid}
                if erset:
                    url = req.build_url(vid='muledit', rql=bookmarksrql)
                    html.append(self.render_action(url, req._('edit bookmarks')))
            # Pick bookmarks
            url = req.user.absolute_url(vid='xaddrelation', rtype='bookmarked_by',
                                        target='subject')
            html.append(self.render_action(url, req._('pick existing bookmarks')))
        # Add divided
        html.append(u'<li class="divider"></li>')
        # Add bookmarks
        for bookmark in self.bookmarks_rset.entities():
            html.append(self.render_action(bookmark.action_url(), bookmark.title))
        # Render box
        ddb = AUSDropDownBox(self._cw._('manage bookmarks'), u'\n'.join(html))
        ddb.render(w=w)



###############################################################################
### DOWNLOAD BOX ##############################################################
###############################################################################
class BrainomicsDownloadBox(component.CtxComponent):
    __select__ = component.CtxComponent.__select__ & nonempty_rset() & is_instance(*ALL_DOWNLOADABLE)
    context = 'left'
    order = 1
    __regid__ = 'ctx-download-box'

    def render(self, w, **kwargs):
        possible_actions = self._cw.vreg['actions'].possible_actions(self._cw, self.cw_rset)
        links = list(possible_actions.get('download-links', []))
        if links:
            w(u'<div class="well admintools">')
            w(u'<h4>%s</h4>' % self._cw._('Download tools'))
            for link in links:
                url = self._cw.build_url(rql=self.cw_rset.printable_rql(), vid=link.download_vid)
                _id = link.__regid__
                w(u'<a class="btn btn-primary btn-block download-ctx" id="%s" href="%s">'
                  % (_id, url))
                w(u'<i class="glyphicon glyphicon-download"></i> %s</a>' % self._cw._(link.title))
            w(u'</div>')
            # Add on load the rql facet change
            self._cw.add_onload("""$(cw).bind('facets-content-loaded',
            cw.cubes.brainomics.changeDownloadUrls);""")


###############################################################################
### DATA BOXES ################################################################
###############################################################################
class BrainomicsResultsFileBox(component.CtxComponent):
    __select__ = (component.CtxComponent.__select__ & nonempty_rset() & one_line_rset()
                  & relation_possible('results_files'))
    context = 'left'
    __regid__ = 'ctx-results-files-box'
    order = 2
    rtype = 'results_files'
    title = _('Results files')

    def render(self, w, **kwargs):
        entity = self.cw_rset.get_entity(0, 0)
        _files = getattr(entity, self.rtype)
        if not _files:
            return
        w(u'<div class="well">')
        w(u'<h4>%s</h4>' % self._cw._(self.title))
        w(u'<ul>')
        for _file in _files:
            title = _file.dc_title().rsplit('/')[-1]
            w(u'<li>%s<a href="%s" title="%s">%s</a></li>'
              % (_file.view('icon'), _file.absolute_url(vid='download'),
                 xml_escape(title), xml_escape(title[:15]+'...')))
        w(u'</ul>')
        w(u'</div>')


class BrainomicsConfigurationFileBox(BrainomicsResultsFileBox):
    __select__ = (component.CtxComponent.__select__ & nonempty_rset() & one_line_rset()
                  & relation_possible('configuration_files'))
    __regid__ = 'ctx-configuration-files-box'
    rtype = 'configuration_files'
    title = _('Configuration files')
    order = 3

def filter_hiddens(w, baserql, wdgs, **kwargs):
    # start brainomincs customization
    for wdg in wdgs:
        # we would probably like to dynamically add other facets elsewhere
        w(u'<input type="hidden" name="facets" value="%s" />' % (
            xml_escape(wdg.facet.__regid__)))
    # end brainomincs customization
    kwargs['baserql'] = baserql
    for key, val in kwargs.items():
        w(u'<input type="hidden" name="%s" value="%s" />' % (
            key, xml_escape(val)))

class BrainomicsFilterBox(facets.FilterBox):
    http_cache_manager = NoHTTPCacheManager
    bk_linkbox_template = u'<p class="btn btn-primary btn-facet">%s</p>'
    __select__ = facets.FilterBox.__select__ | has_dynamic_facets()

    def layout_widgets(self, w, wdgs):
        super(BrainomicsFilterBox, self).layout_widgets(w, wdgs)
        w(u'<div class="admintools">')
        w(u'<div id="dynamic-facet-container"></div>')
        for comp in self._cw.vreg['dynamic-facet'].possible_objects(self._cw, rset=self.cw_rset):
            comp.render(w=w)
        w(u'</div>')

    def clean_dynamic_facets(self):
        """ clean all dynamically created facets.
        """
        vreg = self._cw.vreg['facets']
        facetids = [k for k, v in vreg.iteritems() if v and getattr(v[0], 'dynamic_facet', False)]
        for facetid in facetids:
            vreg.pop(facetid)

    def generate_form(self, w, rset, divid, vid, vidargs=None, mainvar=None,
                      paginate=False, cssclass='', hiddens=None, **kwargs):
        kwargs.pop('context', None)
        # clean dynamic-facet vreg : remove all previously dynamically created facets
        self.clean_dynamic_facets()
        # do not use the facets cache, recompute all facets
        baserql, wdgs = facets._facets(self._cw, rset, context=self.__regid__,
                                       mainvar=mainvar, **kwargs)
        # Remove assert wdgs as now, we may have dynamic facets
        self._cw.add_js(self.needs_js)
        self._cw.add_css(self.needs_css)
        self._cw.html_headers.define_var('facetLoadingMsg',
                                         self._cw._('facet-loading-msg'))
        vidargs = {}
        vidargs = dict((k, v) for k, v in vidargs.iteritems() if v)
        facetargs = xml_escape(json_dumps([divid, vid, paginate, vidargs]))
        w(u'<form id="%sForm" class="%s" method="post" action="" '
          'cubicweb:facetargs="%s" >' % (divid, cssclass, facetargs))
        w(u'<fieldset>')
        if hiddens is None:
            hiddens = {}
        if mainvar:
            hiddens['mainvar'] = mainvar
        filter_hiddens(w, baserql, wdgs, **hiddens)
        self.layout_widgets(w, self.sorted_widgets(wdgs))
        w(u'<input type="submit" class="hidden" />')
        w(u'</fieldset>\n')
        w(u'</form>\n')


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__, (BrainomicsEditBox,
                                                     BrainomicsFilterBox,
                                                     BrainomicsBookmarksBox))
    vreg.register_and_replace(BrainomicsEditBox, EditBox)
    vreg.register_and_replace(BrainomicsBookmarksBox, BookmarksBox)
    vreg.register_and_replace(BrainomicsFilterBox, facets.FilterBox)

    # Unregister breadcrumbs
    from cubicweb.web.views.ibreadcrumbs import (BreadCrumbEntityVComponent,
                                                 BreadCrumbLinkToVComponent,
                                                 BreadCrumbAnyRSetVComponent,
                                                 BreadCrumbETypeVComponent)
    vreg.unregister(BreadCrumbEntityVComponent)
    vreg.unregister(BreadCrumbAnyRSetVComponent)
    vreg.unregister(BreadCrumbETypeVComponent)
    vreg.unregister(BreadCrumbLinkToVComponent)
    # Unregister logo for now...
    # Unregister anon status component
    from cubicweb.web.views.basecomponents import ApplLogo, AnonUserStatusLink
    vreg.unregister(ApplLogo)
    vreg.unregister(AnonUserStatusLink)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import os.path as osp
import tempfile
from contextlib import closing
from zipfile import ZipFile, ZIP_DEFLATED

from cubicweb.predicates import is_instance
from cubicweb.view import EntityView


CSV_DOWNLOADABLE = ('Subject', 'Scan', 'ScoreDefinition', 'QuestionnaireRun',
                    'Answer', 'GenomicMeasure', 'Snp', 'GenericTestRun')
XCEDE_DOWNLOADABLE = ('Subject', 'Scan', 'GenomicMeasure', 'QuestionnaireRun', 'Questionnaire')
JSON_DOWNLOADABLE = ('Subject',)
ZIP_DOWNLOADABLE = ('Scan', 'GenomicMeasure')
ALL_DOWNLOADABLE = set(CSV_DOWNLOADABLE + XCEDE_DOWNLOADABLE + ZIP_DOWNLOADABLE)


###############################################################################
### UTILITY FUNCTIONS #########################################################
###############################################################################
def zipfiles(filepaths):
    """generates a zip archive from `filepaths`

    :param filepaths: is a dictionary mapping basename (as it will appear in the
                      archive) to corresponding absolute filepath (where it must
                      read from on the filesystem)

    :return: a couple fileobj, filepath where `fileobj` is a standard python
             open file objet on the generated archive, and `filepath` is an
             absolute file path to this archive on the filesystem
    """
    fd, archive_filepath = tempfile.mkstemp()
    try:
        noext_archivename = osp.splitext(DataZipView.archive_name)[0]
        with closing(ZipFile(archive_filepath, "w", ZIP_DEFLATED, True)) as zip:
            for filename, subj, _fileobj in filepaths:
                zip.writestr('%s/%s/%s' % (noext_archivename, subj, filename), _fileobj.read())
        fileobj = os.fdopen(fd)
        fileobj.seek(0)
        return fileobj, archive_filepath
    except:
        os.close(fd)
        os.unlink(archive_filepath)
        raise


###############################################################################
### ZIP VIEWS #################################################################
###############################################################################
class DataZipView(EntityView):
    """Abstract base class for the zip view"""
    __regid__ = 'data-zip'
    __select__ = EntityView.__select__ & is_instance(*ZIP_DOWNLOADABLE)
    templatable = False
    binary = True
    archive_name = 'brainomics_data.zip'

    def set_request_content_type(self):
        self._cw.set_content_type('application/zip', filename=self.archive_name)

    def call(self):
        if not self.cw_rset:
            return
        filepaths = set()
        for entity in self.cw_rset.entities():
            for _file in entity.results_files:
                if _file.__regid__ == 'FileSet':
                    for _f in _file.file_entries:
                        filepaths.add(_file.dc_title(), subj, _f)
                else:
                        filepaths.add(_file.dc_title(), subj, _file)
        fileobj, archive_filepath = zipfiles(list(filepaths))
        try:
            self.w(fileobj.read())
        finally:
            fileobj.close()
            os.unlink(archive_filepath)



# -*- coding: utf-8 -*-
# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2013 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from logilab.mtconverter import xml_escape

from cubicweb.predicates import is_instance, nonempty_rset, relation_possible
from cubicweb.web import component


###############################################################################
### TABBLABLE SELECTION COMPONENT #############################################
###############################################################################
class BrainomicsTabblableSelectionComponent(component.CtxComponent):
    """ Component of selection of the different tabs
    """
    __select__ = component.CtxComponent.__select__ & nonempty_rset()
    context = 'nav-toolbar'
    __regid__ = 'tabblable-select-ctx'

    def render(self, w, **kwargs):
        if 'ctx-entity-tabs' in self._cw.vreg:
            tabs = list(self._cw.vreg['ctx-entity-tabs'].possible_objects(self._cw,
                                                                          rset=self.cw_rset))
        else:
            tabs = []
        if tabs:
            w(u'<ul class="nav nav-pills" id="entity-tab">')
            # Main info
            w(u'<li class="active"><a href="#entity-mainview" data-toggle="pill" '
              'class="bnt btn-warning">%s</a></li>' % xml_escape(self._cw._('Main info')))
            # Other tabs
            for tab in tabs:
                w(u'<li><a href="#%s" data-toggle="pill" class="bnt btn-warning">%s</a></li>'
                  % (tab.__regid__, tab.get_title()))
            w(u'</ul>')


class AbstractBrainomicsTabblable(component.EntityCtxComponent):
    """ Component used to display a tab. """
    __abstract__ = True
    __registry__ = 'ctx-entity-tabs'

    def get_title(self):
        return

    def render_content(self, w, **kwargs):
        return

    def render(self, w, **kwargs):
        w(u'<div class="tab-pane" id="%s">' % self.__regid__)
        self.render_content(w)
        w(u'</div>')


###############################################################################
### TABBLABLE COMPONENTS - COMMENT ############################################
###############################################################################
class BrainomicsTabblableComment(AbstractBrainomicsTabblable):
    """ Component used to display a tab with comment. """
    __select__ = AbstractBrainomicsTabblable.__select__ & relation_possible('comments', role='object')
    __regid__ = 'entity-comment-tab'

    def get_title(self):
        return self._cw._('Comments')

    def render_content(self, w, **kwargs):
        entity = self.cw_rset.get_entity(0, 0)
        comp = self._cw.vreg['ctxcomponents'].select_or_none('commentsection',
                                                             self._cw, rset=self.cw_rset,
                                                             entity=entity, __cache=None)
        if comp is not None:
            comp.render(w=w)


###############################################################################
### TABBLABLE COMPONENTS - WIKI ###############################################
###############################################################################
class BrainomicsTabblableWiki(AbstractBrainomicsTabblable):
    """ Component used to display a tab with wiki. """
    __select__ = AbstractBrainomicsTabblable.__select__ & relation_possible('wiki')
    __regid__ = 'entity-wiki-tab'

    def get_title(self):
        return self._cw._('Documentation')

    def render_content(self, w, **kwargs):
        entity = self.cw_rset.get_entity(0, 0)
        card_rset = self._cw.execute('Any X WHERE X is Card, Q wiki X, Q eid %(eid)s',
                                {'eid': entity.eid})
        if card_rset:
            w(self._cw.view('documentation_card', rset=card_rset))
        else:
            w(u'<div>%s</div>' % self._cw._('No documentation available'))


###############################################################################
### TABBLABLE COMPONENTS - TIMELINE ###########################################
###############################################################################
class BrainomicsTabblableTimeline(AbstractBrainomicsTabblable):
    """ Component used to display a tab with a timeline. """
    __select__ = AbstractBrainomicsTabblable.__select__ & is_instance('Subject')
    __regid__ = 'entity-timeline-tab'

    def get_title(self):
        return self._cw._('Timeline')

    def render_content(self, w, **kwargs):
        w(self._cw.view('vtimeline', rset=self.cw_rset))


###############################################################################
### TABBLABLE COMPONENTS - JQPLOT #############################################
###############################################################################
class BrainomicsTabblableJqplotQuestion(AbstractBrainomicsTabblable):
    __select__ = AbstractBrainomicsTabblable.__select__ & is_instance('Question')
    __regid__ = 'entity-jqplot-tab'

    def get_title(self):
        return self._cw._('Plot')

    def get_values_rset(self):
        # XXX Adapter ?
        entity = self.cw_rset.get_entity(0, 0)
        return self._cw.execute('Any V, COUNT(A) GROUPBY V WHERE A is Answer, '
                                'A question Q, Q eid %(e)s, A value V', {'e': entity.eid})

    def render_content(self, w, **kwargs):
        entity = self.cw_rset.get_entity(0, 0)
        rset = self.get_values_rset()
        jsbind = self.add_jsbind(w)
        if rset:
            w(self._cw.view('jqplot-2col', tab_id='pill', rset=rset, jsbind=jsbind,
                            title=xml_escape(entity.dc_title()),
                            displayactions=True, actions=('print',),
                            legend={'show': False}, width='100%'))
        else:
            w(u'<div>%s</div>' % self._cw._('No values available'))

    def get_rql(self):
        entity = self.cw_rset.get_entity(0, 0)
        return 'Any A WHERE A is Answer, A question Q, Q eid %s, A value' % entity.eid

    def add_jsbind(self, w):
        w(u'<div id="jqplot-info"></div>')
        jsbind = """function (ev, seriesIndex, pointIndex, data) {
        var htmlstr = data[0] + ' : ' + data[1] + ' entries';
        var url = '%s?rql=' + '%s ' + data[0];
        var htmlstr = htmlstr + '   <a href="' + url +'">See all</a>';
        $('#jqplot-info').html(htmlstr);
        }""" % (self._cw.base_url(), self.get_rql())
        return jsbind


class BrainomicsTabblableJqplotScoreDef(BrainomicsTabblableJqplotQuestion):
    __select__ = AbstractBrainomicsTabblable.__select__ & is_instance('ScoreDefinition')

    def get_values_rset(self):
        # XXX Adapter ?
        entity = self.cw_rset.get_entity(0, 0)
        if entity.type == 'numerical':
            rset = self._cw.execute('Any V, COUNT(S) GROUPBY V WHERE S value V, '
                                    'S is ScoreValue, S definition D, D eid %(e)s',
                                    {'e': entity.eid})
        else: # Text
            rset = self._cw.execute('Any V, COUNT(S) GROUPBY V WHERE S text V, '
                                    'S is ScoreValue, S definition D, D eid %(e)s',
                                    {'e': entity.eid})
        return rset

    def get_rql(self):
        entity = self.cw_rset.get_entity(0, 0)
        if entity.type == 'numerical':
            return 'Any S WHERE S is ScoreValue, S definition D, D eid %s, S value' % entity.eid
        else: # Text
            return 'Any S WHERE S is ScoreValue, S definition D, D eid %s, S text' % entity.eid

    def add_jsbind(self, w):
        w(u'<div id="jqplot-info"></div>')
        entity = self.cw_rset.get_entity(0, 0)
        if entity.type == 'numerical':
            jsbind = """function (ev, seriesIndex, pointIndex, data) {
            var htmlstr = data[0] + ' : ' + data[1] + ' entries';
            var url = '%s?rql=' + '%s ' + data[0];
            var htmlstr = htmlstr + '   <a href="' + url +'">See all</a>';
            $('#jqplot-info').html(htmlstr);
            }""" % (self._cw.base_url(), self.get_rql())
        else: # Text
            jsbind = """function (ev, seriesIndex, pointIndex, data) {
            var htmlstr = ticks[pointIndex] + ' : ' + data[1] + ' entries';
            var url = '%s?rql=' + '%s "' + ticks[pointIndex] + '"';
            console.log(url)
            var htmlstr = htmlstr + "   <a href='" + url +"'>See all</a>";
            $('#jqplot-info').html(htmlstr);
            }""" % (self._cw.base_url(), self.get_rql())
        return jsbind


###############################################################################
### TABBLABLE COMPONENTS - ASSESSMENTS ########################################
###############################################################################
class BrainomicsTabblableAssessmentView(AbstractBrainomicsTabblable):
    __select__ = AbstractBrainomicsTabblable.__select__ & is_instance('Subject')
    __regid__ = 'subject-assessment-tab'

    def get_title(self):
        return self._cw._('Assessments')

    def render_content(self, w, **kwargs):
        entity = self.cw_rset.get_entity(0, 0)
        rset = self._cw.execute('Any X WHERE S concerned_by X, S eid %(e)s', {'e': entity.eid})
        if rset:
            w(self._cw.view('sameetypelist', rset=rset))


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__,
                      (BrainomicsTabblableTimeline,))
    if 'vtimeline' in vreg.config.cubes():
        vreg.register(BrainomicsTabblableTimeline)



# -*- coding: utf-8 -*-
# copyright 2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# copyright 2014 CEA (Saclay, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
from cubicweb.predicates import is_instance
from cubicweb.view import EntityView


class BrainomicsAbstractOutOfContextView(EntityView):
    __regid__ = 'outofcontext'
    __abstract__ = True
    icon = None
    local_image = None

    def add_icon(self):
        if self.icon:
            icon = u'<span class="glyphicon glyphicon-%s"></span>' % self.icon
        elif self.local_image:
            image = u'<img alt="" src="%s">' %  self._cw.data_url(self.local_image)
        else:
            icon = u''
        self.w(u'<div class="col-md-2"><p class="text-center">%s</p></div>' % icon)

    def additional_infos(self, entity):
        pass

    def cell_call(self, row, col):
        w = self.w
        entity = self.cw_rset.get_entity(row, col)
        w(u'<div class="ooview"><div class="well"><div class="row">')
        self.add_icon()
        w(u'<div class="col-md-10"><h4>%s</h4></div>' % entity.view('incontext'))
        w(u'</div>')
        self.additional_infos(entity)
        w(u'</div></div>')


class StudyOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Study')
    icon = 'folder-open'


class SubjectOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Subject')
    icon = 'user'


class AssessmentOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Assessment')
    icon = 'time'


class GenomicMeasureOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('GenomicMeasure')
    icon = 'filter'


class GeneContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Gene')
    icon = 'link'


class GenomicRegionContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('GenomicRegion')
    icon = 'pushpin'


class ScanOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Scan')
    icon = 'film'


class QuestionnaireOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Questionnaire')
    icon = 'list'


class QuestionnaireRunOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('QuestionnaireRun')
    icon = 'list-alt'


class AnswerOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Answer')
    icon = 'info-sign'


class QuestionContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Question')
    icon = 'question-sign'


class ScoreDefinitionOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('ScoreDefinition')
    icon = 'tags'


class ScoreValueOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('ScoreValue')
    icon = 'tag'


class TherapyOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Therapy')
    icon = 'ok-sign'


class DiagnosticOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Diagnostic')
    icon = 'comment'


class CenterOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Center')
    icon = 'home'


class MedicalTechniqueOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('MedicalTechnique')
    icon = 'cog'


class DiseaseOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Disease')
    icon = 'certificate'


class BodyLocationOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('BodyLocation')
    icon = 'heart-empty'


class DrugContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Drug')
    icon = 'tint'


class DrugTakeContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('DrugTake')
    icon = 'calendar'


class FileOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('ExternalFile', 'File')
    icon = 'file'


class FileSetOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('FileSet')
    icon = 'folder-close'


class GenericTestRunOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('GenericTestRun')
    icon = 'wrench'


class BiologicalSamplesOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('Biopsy',
                                                                             'BiopsySample',
                                                                             'CellCulture',
                                                                             'BloodSample')
    icon = 'asterisk'


class TherapyEvaluationOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('TherapyEvaluation')
    icon = 'info-sign'


class SurvivalDataOutOfContextView(BrainomicsAbstractOutOfContextView):
    __select__ = BrainomicsAbstractOutOfContextView.__select__ & is_instance('SurvivalData')
    icon = 'heart'



# copyright 2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

import csv
import json
import pickle
import re
from itertools import chain


def get_image_info(image_path, get_tr=True):
    """Return the image info readed from the image using Nibabel"""
    import nibabel as nb
    img = nb.load(image_path)
    data = {}
    data['voxel_res_x'] = float(img.get_header()['pixdim'][1])
    data['voxel_res_y'] = float(img.get_header()['pixdim'][2])
    data['voxel_res_z'] = float(img.get_header()['pixdim'][3])
    data['shape_x'] = int(img.get_shape()[0])
    data['shape_y'] = int(img.get_shape()[1])
    data['shape_z'] = int(img.get_shape()[2])
    data['shape_t'] = int(img.get_shape()[3]) if len(img.get_shape()) == 4 else None
    data['affine'] = pickle.dumps(img.get_affine().tolist())
    desc = str(img.get_header()['descrip'])
    # Use desc ?
    try:
        if get_tr:
            tr, te = re.findall(
                'TR=(.*)ms.*TE=(.*)ms', desc)[0]
            data['tr'] = float(tr)
            data['te'] = float(te)
    except Exception, e:
        data['tr'] = None
        data['te'] = None

    return data


def import_genes(ref_chr_path, ref_gene_path):
    """Import genes"""
    chromosomes = json.load(open(ref_chr_path))
    ref_gene = []
    for row in csv.reader(open(ref_gene_path), delimiter='\t'):
        gene = {}
        gene['name'] = unicode(row[0])
        gene['gene_id'] = unicode(row[0])
        gene['uri'] = None
        gene['start_position'] = int(row[1])
        gene['stop_position'] = int(row[2])
        gene['chromosome'] = row[3].split('_')[0]
        ref_gene.append(gene)
    return ref_gene


def import_chromosomes(ref_chr_path):
    """Import chromosomes"""
    chromosomes = json.load(open(ref_chr_path))
    chrs = []
    for chr_id in chromosomes:
        chr = {}
        chr['name'] = u'chr%s' % chromosomes[chr_id].upper()
        chr['identifier'] = unicode(chr['name'])
        chrs.append(chr)
    return chrs


def import_snps(ref_chr_path, ref_snp_path):
    """Import snps"""
    chromosomes = json.load(open(ref_chr_path))
    snps = []
    for row in csv.reader(open(ref_snp_path), delimiter='\t'):
        snp = {}
        if row[0] == '0':
            continue
        chr_id = chromosomes[row[0]]
        snp['rs_id'] = unicode(row[1])
        snp['position'] = int(row[3])
        snp['chromosome'] = u'chr%s' % chr_id.upper()
        snps.append(snp)
    return snps


def generate_container_relations(session, cursor=None):
    """ Generate the relations required for container """
    # XXX Get the cursor of on the current cnx for sqlite tests...
    cursor = session.system_sql('SELECT * FROM cw_cwetype LIMIT 1')
    for rtype, rset in chain(_generate_of_subject_relations(session),
                             _generate_in_assessment_relations(session)):
        _push_container_relations(cursor, rtype, rset)
    session.commit()


def _push_container_relations(cursor, rtype, rset):
    """ Push the relations required by container """
    data = {}
    for meid, aeid, etype in rset:
        data.setdefault(etype,  []).append({'e': meid, 'a': aeid})
    for etype, eids in data.iteritems():
        cursor.executemany('UPDATE cw_%s SET cw_%s=%%(a)s WHERE cw_eid=%%(e)s'
                           % (etype.lower(), rtype), eids)


def _generate_in_assessment_relations(session):
    """ Generate the 'in_assessment' relations used for container """
    # Measures
    rset = session.execute('Any X, A, N WITH X, A, N BEING ('
                           '(Any X,A,N WHERE X is E, E name N, A generates X) '
                           'UNION (Any X,A,N WHERE X is E, E name N, A uses X))')
    yield ('in_assessment', rset)
    # Files
    rset = session.execute('Any X, A, N WITH X, A, N BEING ('
                           '(Any X,A,N WHERE X is E, E name N, M configuration_files X, '
                           'M in_assessment A) '
                           'UNION (Any X,A,N WHERE X is E, E name N, M results_files X, '
                           'M in_assessment A))')
    yield ('in_assessment', rset)
    # Measures relations
    for rtype in ('generates', 'uses'):
        rset = session.execute('Any X, A, N WITH X, A, N BEING ('
                               '(Any X,A,N WHERE X is E, E name N, X measure M, A %(r)s M) '
                               'UNION (Any X,A,N WHERE X is E, E name N, X comments M, '
                               'A %(r)s M))' % {'r': rtype})
        yield ('in_assessment', rset)
        # Assessment Files
        rset = session.execute('Any X, A, N WITH X, A, N BEING ('
                               '(Any X,A,N WHERE X is E, E name N, A %(r)s M, '
                               'M configuration_files X) '
                               'UNION (Any X,A,N WHERE X is E, E name N, A %(r)s M, '
                               'M results_files X))'
                                % {'r': rtype})
        yield ('in_assessment', rset)


def _generate_of_subject_relations(session):
    """ Generate the 'of_subject' relations used for container """
    # Diagnostic, Therapy
    rset = session.execute('Any X, A, N WITH X, A, N BEING ('
                           '(Any X, A, N WHERE A related_diagnostics X, X is E, E name N) '
                           'UNION (Any X, A, N WHERE A related_therapies X, X is E, E name N))')
    yield ('of_subject', rset)
    # Drug take
    rset = session.execute('Any X, A, N WITH X, A, N BEING ('
                           'Any X, A, N WHERE A related_therapies T, '
                           'X taken_in_therapy T, X is E, E name N)')
    yield ('of_subject', rset)
    # Admission
    rset = session.execute('Any X, A, N WITH X, A, N BEING ('
                           'Any X, A, N WHERE X admission_of A, X is E, E name N)')
    yield ('of_subject', rset)
    # Admission step
    rset = session.execute('Any X, A, N WITH X, A, N BEING ('
                           'Any X, A, N WHERE S admission_of A, X step_of S, X is E, E name N)')
    yield ('of_subject', rset)
    # Add samples
    rset = session.execute('Any X, A, N WITH X, A, N BEING ('
                           'Any X, A, N WHERE A related_samples X, X is E, E name N)')
    yield ('of_subject', rset)



# pylint: disable=W0622
"""cubicweb-brainomics application packaging information"""

modname = 'brainomics'
distname = 'cubicweb-brainomics'

numversion = (0, 11, 0)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
author = 'LOGILAB S.A. (Paris, FRANCE)'
author_email = 'contact@logilab.fr'
description = 'Cube for the Brainomics Project, see http://www.brainomics.net/'
web = 'http://www.cubicweb.org/project/%s' % distname

__depends__ =  {'cubicweb': '>= 3.18.5',
                'cubicweb-questionnaire': '>= 0.7.0',
                'cubicweb-neuroimaging': '>= 0.7.0',
                'cubicweb-genomics': '>= 0.10.0',
                'cubicweb-medicalexp': '>= 0.12.0',
                'cubicweb-card': None,
                'cubicweb-bootstrap': '>= 0.6.0',
                'cubicweb-squareui': '>= 0.3.5',
                'cubicweb-comment': None,
                'cubicweb-jqplot': None,
                'cubicweb-clinipath': '>= 0.2.0',
                }
__recommends__ = {'cubicweb-vtimeline': '>= 0.5.0'}


from os import listdir as _listdir
from os.path import join, isdir
from glob import glob

THIS_CUBE_DIR = join('share', 'cubicweb', 'cubes', modname)

def listdir(dirpath):
    return [join(dirpath, fname) for fname in _listdir(dirpath)
            if fname[0] != '.' and not fname.endswith('.pyc')
            and not fname.endswith('~')
            and not isdir(join(dirpath, fname))]

data_files = [
    # common files
    [THIS_CUBE_DIR, [fname for fname in glob('*.py') if fname != 'setup.py']],
    ]
# check for possible extended cube layout
for dname in ('entities', 'views', 'sobjects', 'hooks', 'schema', 'data', 'wdoc',
              'i18n', 'migration', 'importers',
              'data/bootstrap', 'data/bootstrap/css', 'data/bootstrap/img', 'data/images', 'data/img',):
    if isdir(dname):
        data_files.append([join(THIS_CUBE_DIR, dname), listdir(dname)])
# Note: here, you'll need to add subdirectories if you want
# them to be included in the debian package

